# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Corporation, All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.core.urlresolvers import reverse
from django import http

from mox import IsA  # noqa

from openstack_dashboard import api
from openstack_dashboard.test import helpers as test
from openstack_dashboard.usage import quotas

INDEX_URL = reverse('horizon:vsm:rbdpools:index')

class RbdPoolsTests(test.TestCase):
    @test.create_stubs({api.vsm: ('pool_list',
                                     'rbd_status',),
                        })
    def test_index(self):
        pools = self.storage_pools.list()
        rbd_status = []
        api.vsm.pool_list(IsA(http.HttpRequest)).AndReturn(pools)
        api.vsm.rbd_status(IsA(http.HttpRequest)).AndReturn(rbd_status)

        api.cinder.volume_list(IsA(http.HttpRequest), search_opts=None).\
            AndReturn(volumes)
        api.nova.server_list(IsA(http.HttpRequest), search_opts=None).\
            AndReturn([self.servers.list(), False])
        api.cinder.volume_snapshot_list(IsA(http.HttpRequest)).\
            AndReturn(vol_snaps)
        api.cinder.volume_list(IsA(http.HttpRequest)).AndReturn(volumes)
        quotas.tenant_quota_usages(IsA(http.HttpRequest)).MultipleTimes(). \
            AndReturn(self.quota_usages.first())
        self.mox.ReplayAll()

        res = self.client.get(INDEX_URL)
        self.assertEqual(res.status_code, 200)
        self.assertTemplateUsed(res, 'project/volumes/index.html')




# Copyright 2014 Intel Corporation, All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import logging

from horizon import tables
from .base import BaseUsage

LOG = logging.getLogger(__name__)

class UsageView(tables.DataTableView):
    usage_class = None
    show_terminated = True

    def __init__(self, *args, **kwargs):
        super(UsageView, self).__init__(*args, **kwargs)
        if not issubclass(self.usage_class, BaseUsage):
            raise AttributeError("You must specify a usage_class attribute "
                                 "which is a subclass of BaseUsage.")

    def get_template_names(self):
        if self.request.GET.get('format', 'html') == 'csv':
            return ".".join((self.template_name.rsplit('.', 1)[0], 'csv'))
        return self.template_name

    def get_content_type(self):
        if self.request.GET.get('format', 'html') == 'csv':
            return "text/csv"
        return "text/html"

    def get_data(self):
        tenant_id = self.kwargs.get('tenant_id', self.request.user.tenant_id)
        self.usage = self.usage_class(self.request, tenant_id)
        self.usage.summarize(*self.usage.get_date_range())
        self.usage.get_quotas()
        self.kwargs['usage'] = self.usage
        return self.usage.usage_list

    def get_context_data(self, **kwargs):
        context = super(UsageView, self).get_context_data(**kwargs)
        context['table'].kwargs['usage'] = self.usage
        context['form'] = self.usage.form
        context['usage'] = self.usage
        return context

    def render_to_response(self, context, **response_kwargs):
        resp = self.response_class(request=self.request,
                                   template=self.get_template_names(),
                                   context=context,
                                   content_type=self.get_content_type(),
                                   **response_kwargs)
        if self.request.GET.get('format', 'html') == 'csv':
            resp['Content-Disposition'] = 'attachment; filename=usage.csv'
            resp['Content-Type'] = 'text/csv'
        return resp



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Corporation, All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
from django.utils.translation import ugettext_lazy as _
from django.views.decorators.csrf import csrf_exempt
from django.shortcuts import render
from django import forms as django_froms
from horizon import exceptions
from horizon import tables
from horizon import forms
from vsm_dashboard.api import vsm as vsmapi
from .tables import OsdsTable
from django.http import HttpResponse,HttpResponseRedirect

import json
LOG = logging.getLogger(__name__)

def device_get_smartinfo(request,device_id=None,device_path=None):
    search_opts = {}
    print 'device_id====',device_id,'--device_path=',device_path
    search_opts['device_id'] = device_id
    search_opts['device_path'] = device_path
    ret = vsmapi.device_get_smartinfo(request,search_opts=search_opts)
    return ret

class IndexView(tables.DataTableView):
    table_class = OsdsTable
    template_name = 'vsm/devices-management/index.html'

    def get_data(self):
        default_limit = 10000;
        default_sort_dir = "asc";
        default_sort_keys = ['osd_name']
        marker = self.request.GET.get('marker', "")
        osd_id = self.request.GET.get('osdid',"-1")
        try:
            _osds = vsmapi.osd_status(self.request, paginate_opts={
                "limit": default_limit,
                "sort_dir": default_sort_dir,
                "marker":   marker,
            })
        except:
            _osds = []
            exceptions.handle(self.request,
                              _('Unable to retrieve osds. '))
        if _osds:
            logging.debug("resp osds in view: %s" % _osds)
        osds = []
        settings = vsmapi.get_setting_dict(self.request)
        if 'disk_near_full_threshold' not in settings.keys():
            vsmapi.update_setting(self.request,'disk_near_full_threshold','75')
        if 'disk_full_threshold' not in settings.keys():
            vsmapi.update_setting(self.request,'disk_full_threshold','85')
        disk_near_full_threshold = int(settings.get('disk_near_full_threshold',75))
        disk_full_threshold = int(settings.get('disk_full_threshold',85))
        for _osd in _osds:
            osd = {
                    'id': _osd.id,
                    'osd': _osd.osd_name,
                    'vsm_status': _osd.operation_status,
                    'osd_state': _osd.state,
                    'osd_weight': _osd.weight,
                    'storage_group': _osd.device['device_type'],
                    'data_dev_path': _osd.device['path'],
                    'data_dev_status': _osd.device['state'],
                    'data_dev_capacity': 0 if not _osd.device['total_capacity_kb']\
            else int(_osd.device['total_capacity_kb']/1024),
                    'data_dev_used': 0 if not _osd.device['used_capacity_kb']\
            else int(_osd.device['used_capacity_kb']/1024),
                    'data_dev_available': 0 if not _osd.device['avail_capacity_kb']\
            else int(_osd.device['avail_capacity_kb']/1024),
                    'journal_device_path': _osd.device['journal'],
                    'journal_device_status': _osd.device['journal_state'], 
                    'server': _osd.service['host'],
                    'zone': _osd.zone,
                    'full_warn': 0
                  }

            if osd['data_dev_capacity']:
                osd['full_status'] = round(osd['data_dev_used'] * 1.0 / osd['data_dev_capacity'] * 100, 2)
                if osd['full_status'] >= disk_near_full_threshold and osd['full_status'] < disk_full_threshold:
                    osd['full_warn'] = 1
                elif osd['full_status'] >=disk_full_threshold:
                    osd['full_warn'] = 2
            else:
                osd['full_status'] = ''
            if osd_id == "-1":
                osds.append(osd)  #all of the deivces
            elif str(_osd.id) == str(osd_id):
                osds.append(osd)  #filter the devices by osd_id
            else:
                pass
        return osds

@csrf_exempt
def add_new_osd(request):
    template = "vsm/devices-management/add_osd.html"
    context = {}
    #get the server list
    servers = vsmapi.get_server_list(None, )
    context["servers"] = servers
    context["upload_file"] = UploadFileForm()
    context["storage_group"] = get_storage_group_list(request)
    context["osd_location"] = get_osd_location_list(request)

    #get the osd list from the file
    if request.method == "POST":
        form = UploadFileForm(request.POST,request.FILES)
        if form.is_valid():
            context["osd_list"] = handle_uploaded_file(request.FILES['file'])
            print 10 * "="
            print "the import osd list"
            print context["osd_list"]
            return render(request,template,context)
    return render(request,template,context)


def add_new_osd_action(request):
    data = json.loads(request.body)
    print 'data----7777==',data
    #vsmapi.add_new_disks_to_cluster(request,data)
    vsmapi.add_batch_new_disks_to_cluster(request,data)
    status_json = {"status":"OK"}
    status_data = json.dumps(status_json)
    return HttpResponse(status_data)

def get_storage_group_list(request):
    #get the storage group
    storage_group_list = vsmapi.storage_group_status(request)
    storage_group = []
    for _sg in storage_group_list:
        sg = {
            "id":_sg.id
            ,"name":_sg.name
        }
        storage_group.append(sg)
    return storage_group

def get_osd_location_list(request):
    zones = vsmapi.osd_locations_choices_by_type(request)
    osd_location_list = zones['osd_locations_choices']
    location_list = []
    print 'osd_location_list===',osd_location_list
    for location in osd_location_list:
        location_dict = {
            "id":location['id'],
            "name":location['name'],
        }
        location_list.append(location_dict)
    print 'location_list===',location_list
    return location_list

def get_osd_list_data(service_id):
    osd_list = vsmapi.osd_status(None, paginate_opts={
        "service_id": int(service_id)
    })
    return osd_list

class UploadFileForm(django_froms.Form):
    file = forms.FileField()

def handle_uploaded_file(f):
    osd_list = []
    try:
        #get the file content
        file_content = []
        for chunk in f.chunks():
           file_content.append(chunk)
        file_content = file_content[0].replace("\r\n",";").replace("\n",";").split(";");
        osd_list = generate_import_data_format(file_content)
    except Exception, e:
        print e
    return osd_list

def generate_import_data_format(file_content):
    osd_list = []
    for i,item in enumerate(file_content):
        #ignore the first line, there is no data
        if i == 0:
            continue
        #get the osd list
        osd = item.split(",")
        if len(osd) != 1:
            osd = {
                "server_name":osd[0],
                "sg_name":osd[1],
                "weight":osd[2],
                "journal":osd[3],
                "data":osd[4],
                "location":osd[5]
            }
            osd_list.append(osd)
    return osd_list



def DevicesAction(request, action):
    data = json.loads(request.body)

    device_data_str = device_get_smartinfo(request,str(data["osd_id"]))
    device_data_dict = {}
    device_data_str = device_data_str[0].device_data
    if device_data_str == None:
        devicedata = json.dumps({"data":""})
        return HttpResponse(devicedata)

    for data in device_data_str.split("\n"):
        data_list = data.split("=")
        data_list_len = len(data_list)
        if data_list_len == 1:
            device_data_dict[data_list[0]] = ""
        if data_list_len == 2:
            device_data_dict[data_list[0]] = data_list[1]
            device_data_json = {
                "basic":{"status":device_data_dict["Drive Status"],
                       "family":device_data_dict["Drive Family"],
                       "seriesNumber":device_data_dict["Serial Number"],
                       "firmware":device_data_dict["Firmware"],
                       "totalCapacity":device_data_dict["Capacity in total"],
                       "usedCapacity":device_data_dict["Capacity in use"],
                },
                "smart":{"percentageUsed":device_data_dict["Percentage Used"],
                       "temperature":device_data_dict["Temperature"],
                       "unitRead":str(int(device_data_dict["Data Units Read"],16)),
                       "unitWRITE":device_data_dict["Data Units Written"],
                }
            }

    devicedata = json.dumps(device_data_json)
    return HttpResponse(devicedata)


def get_smart_info(request):
    data = json.loads(request.body)

    device_data_dict = device_get_smartinfo(request,str(data["osd_id"]),data["device_path"])
    device_data_json = {
               "basic":{
                    "DriveFamily":device_data_dict["basic"].get("Drive Family")
                   ,"SerialNumber":device_data_dict["basic"].get("Serial Number")
                   ,"FirmwareVersion":device_data_dict["basic"].get("Firmware Version")
                   ,"DriveStatus":device_data_dict["basic"].get("Drive Status")
                }
                ,"smart":[]
    }

    for _smart_item in device_data_dict["smart"].iteritems():
        _item = {"key":_smart_item[0],"value":_smart_item[1]}
        device_data_json["smart"].append(_item)

    devicedata = json.dumps(device_data_json)
    return HttpResponse(devicedata)


def get_osd_list(request):
    data = json.loads(request.body)
    service_id = int(data["service_id"])
    osds = vsmapi.osd_status(None, paginate_opts={
                "service_id": service_id
            })

    osd_list = []
    for _osd in osds:
        osd = {
            "id":_osd.id
            ,"name":_osd.osd_name
            ,"weight":_osd.weight
            ,"storage_group":_osd.device['device_type']
            ,"journal":_osd.device['journal']
            ,"device":_osd.device['path']
        }
        print _osd
        osd_list.append(osd)


    osd_list_data_json = {"osdlist":osd_list}
    osd_list_data = json.dumps(osd_list_data_json)
    return HttpResponse(osd_list_data)


def check_device_path(request):
    search_opts = json.loads(request.body)
    server_id = search_opts["server_id"]
    data_device_path = search_opts["data_device_path"]
    journal_device_path = search_opts["journal_device_path"]
    if data_device_path == journal_device_path:
        status_json = {"status":"Failed",'message':'data_device_path and journal_device_path can not be the same hard disk'}
    else:
        ret = vsmapi.get_available_disks(request, search_opts={
                    "server_id": server_id
                    ,"path":[data_device_path,journal_device_path]
                })


        if ret["ret"] == 1 :
            status_json = {"status":"OK"}
        else:
            status_json = {"status":"Failed",'message':ret.get('message')}

    status_data = json.dumps(status_json)
    return HttpResponse(status_data)


def restart_osd(request):
    data = json.loads(request.body)
    osd_id_list = data["osd_id_list"]

    for osd_id in osd_id_list:
        vsmapi.osd_restart(request, osd_id)

    rs = json.dumps({"status":0})
    return HttpResponse(rs)

def get_available_disks(request):
    search_opts = json.loads(request.body)
    server_id = search_opts["server_id"]

    ret = vsmapi.get_available_disks(request, search_opts={
                "server_id": server_id
                ,"result_mode":"get_disks",
            })

    disks = ret['disks']
    if disks is None:disks=[]
    disk_data = json.dumps(disks)

    return HttpResponse(disk_data)

def remove_osd(request):
    data = json.loads(request.body)
    osd_id_list = data["osd_id_list"]

    for osd_id in osd_id_list:
        vsmapi.osd_remove(request, osd_id)

    rs = json.dumps({"status":0})
    return HttpResponse(rs)


def restore_osd(request):
    data = json.loads(request.body)
    osd_id_list = data["osd_id_list"]
    print osd_id_list

    for osd_id in osd_id_list:
        vsmapi.osd_restore(request, osd_id)

    rs = json.dumps({"status":0})
    return HttpResponse(rs)



# Copyright 2014 Intel Corporation, All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

from collections import defaultdict
import itertools

from horizon import exceptions
from horizon.utils.memoized import memoized

from vsm_dashboard.api import nova, vsm, network
from vsm_dashboard.api.base import is_service_enabled, QuotaSet

NOVA_QUOTA_FIELDS = ("metadata_items",
                     "cores",
                     "instances",
                     "injected_files",
                     "injected_file_content_bytes",
                     "ram",
                     "floating_ips",
                     "fixed_ips",
                     "security_groups",
                     "security_group_rules",)

CINDER_QUOTA_FIELDS = ("volumes",
                       "gigabytes",)

QUOTA_FIELDS = NOVA_QUOTA_FIELDS + CINDER_QUOTA_FIELDS

class QuotaUsage(dict):
    """ Tracks quota limit, used, and available for a given set of quotas."""

    def __init__(self):
        self.usages = defaultdict(dict)

    def __getitem__(self, key):
        return self.usages[key]

    def __setitem__(self, key, value):
        raise NotImplemented("Directly setting QuotaUsage values is not "
                             "supported. Please use the add_quota and "
                             "tally methods.")

    def __repr__(self):
        return repr(dict(self.usages))

    def add_quota(self, quota):
        """ Adds an internal tracking reference for the given quota. """
        if quota.limit is None or quota.limit == -1:
            # Handle "unlimited" quotas.
            self.usages[quota.name]['quota'] = float("inf")
            self.usages[quota.name]['available'] = float("inf")
        else:
            self.usages[quota.name]['quota'] = int(quota.limit)

    def tally(self, name, value):
        """ Adds to the "used" metric for the given quota. """
        value = value or 0  # Protection against None.
        # Start at 0 if this is the first value.
        if 'used' not in self.usages[name]:
            self.usages[name]['used'] = 0
        # Increment our usage and update the "available" metric.
        self.usages[name]['used'] += int(value)  # Fail if can't coerce to int.
        self.update_available(name)

    def update_available(self, name):
        """ Updates the "available" metric for the given quota. """
        available = self.usages[name]['quota'] - self.usages[name]['used']
        if available < 0:
            available = 0
        self.usages[name]['available'] = available

def _get_quota_data(request, method_name, disabled_quotas=None,
                    tenant_id=None):
    quotasets = []
    if not tenant_id:
        tenant_id = request.user.tenant_id
    quotasets.append(getattr(nova, method_name)(request, tenant_id))
    qs = QuotaSet()
    if disabled_quotas is None:
        disabled_quotas = get_disabled_quotas(request)
    if 'volumes' not in disabled_quotas:
        quotasets.append(getattr(vsm, method_name)(request, tenant_id))
    for quota in itertools.chain(*quotasets):
        if quota.name not in disabled_quotas:
            qs[quota.name] = quota.limit
    return qs

def get_default_quota_data(request, disabled_quotas=None, tenant_id=None):
    return _get_quota_data(request,
                           "default_quota_get",
                           disabled_quotas=disabled_quotas,
                           tenant_id=tenant_id)

def get_tenant_quota_data(request, disabled_quotas=None, tenant_id=None):
    return _get_quota_data(request,
                           "tenant_quota_get",
                           disabled_quotas=disabled_quotas,
                           tenant_id=tenant_id)

def get_disabled_quotas(request):
    disabled_quotas = []
    if not is_service_enabled(request, 'volume'):
        disabled_quotas.extend(CINDER_QUOTA_FIELDS)
    return disabled_quotas

@memoized
def tenant_quota_usages(request):
    # Get our quotas and construct our usage object.
    disabled_quotas = get_disabled_quotas(request)

    usages = QuotaUsage()
    for quota in get_tenant_quota_data(request,
                                       disabled_quotas=disabled_quotas):
        usages.add_quota(quota)

    # Get our usages.
    floating_ips = network.tenant_floating_ip_list(request)
    flavors = dict([(f.id, f) for f in nova.flavor_list(request)])
    instances = nova.server_list(request)
    # Fetch deleted flavors if necessary.
    missing_flavors = [instance.flavor['id'] for instance in instances
                       if instance.flavor['id'] not in flavors]
    for missing in missing_flavors:
        if missing not in flavors:
            try:
                flavors[missing] = nova.flavor_get(request, missing)
            except:
                flavors[missing] = {}
                exceptions.handle(request, ignore=True)

    usages.tally('instances', len(instances))
    usages.tally('floating_ips', len(floating_ips))

    if 'volumes' not in disabled_quotas:
        volumes = vsm.volume_list(request)
        usages.tally('gigabytes', sum([int(v.size) for v in volumes]))
        usages.tally('volumes', len(volumes))

    # Sum our usage based on the flavors of the instances.
    for flavor in [flavors[instance.flavor['id']] for instance in instances]:
        usages.tally('cores', getattr(flavor, 'vcpus', None))
        usages.tally('ram', getattr(flavor, 'ram', None))

    # Initialise the tally if no instances have been launched yet
    if len(instances) == 0:
        usages.tally('cores', 0)
        usages.tally('ram', 0)

    return usages




# Copyright 2014 Intel Corporation, All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

from django.conf.urls import patterns, url
from .views import IndexView,CreateView
from .views import CreateErasureCodedPoolView
from .views import AddCacheTierView
from .views import RemoveCacheTierView
from .views import add_cache_tier,remove_cache_tier,create_replicated_pool,create_ec_pool
from .views import get_default_pg_number_storage_group

urlpatterns = patterns('',
    url(r'^$', IndexView.as_view(), name='index'),
    url(r'^create/$', CreateView.as_view(), name='create'),
    url(r'^create_ec_pool/$', CreateErasureCodedPoolView.as_view(), name='create_ec_pool'),
    url(r'^add_cache_tier/$', AddCacheTierView.as_view(), name='add_cache_tier'),
    url(r'^remove_cache_tier/$', RemoveCacheTierView.as_view(), name='remove_cache_tier'),
    url(r'^add_cache_tier_action/$', add_cache_tier, name='add_cache_tier_action'),
    url(r'^remove_cache_tier_action/$', remove_cache_tier, name='remove_cache_tier_action'),
    url(r'^create_replicated_pool_action/$', create_replicated_pool, name='create_replicated_pool_action'),
    url(r'^create_ec_pool_action/$', create_ec_pool, name='create_ec_pool_action'),
    url(r'^get_default_pg_number_storage_group/$', get_default_pg_number_storage_group, name='get_default_pg_number_storage_group'),
)





# Copyright 2014 Intel Corporation, All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

from django.utils.translation import ugettext_lazy as _

import horizon

class Dashboard(horizon.PanelGroup):
    slug = 'dashboard'
    name = _("Dashboard")
    panels = ('overview',)

class ClusterMgmt(horizon.PanelGroup):
    slug = "clustermgmt"
    name = _("Cluster Management")
    panels = ('clustermgmt', 'cluster-import', 'cephupgrade', 'poolsmanagement','ecprofiles-management','crushmap','zonemgmt')

class ClusterMonitor(horizon.PanelGroup):
    slug = "clustermonitor"
    name = _("Cluster Monitoring")
    panels = ('storage-group-status', 'pool-status', 'osd-status', 'monitor-status', \
              'mds-status', 'pg-status', 'rbd-status')

class ServerMgmt(horizon.PanelGroup):
    slug = "servermgmt"
    name = _("Server Management")
    panels = ('storageservermgmt', 'devices-management',)

class OpenstackMgmt(horizon.PanelGroup):
    slug = "openstackmgmt"
    name = _("OpenStack Integration")
    panels = ('rbdpools', 'openstackconnect',)

class UserMgmt(horizon.PanelGroup):
    slug = "usermgmt"
    name = _("VSM Management")
    panels = ("usermgmt", 'settings')

class VizDash(horizon.Dashboard):
    name = _("VSM")
    slug = "vsm"
    panels = (Dashboard, ServerMgmt, ClusterMgmt, ClusterMonitor, OpenstackMgmt, UserMgmt)
    default_panel = 'overview'
    roles = ('admin',)

horizon.register(VizDash)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Corporation, All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse_lazy
from django.views.generic import TemplateView

from horizon import exceptions
from horizon import tables
from horizon import forms

from vsm_dashboard.api import vsm as vsmapi
from .form import CreateErasureCodedPool
from .form import RemoveCacheTier
from .tables import ListPoolTable

from vsm_dashboard.api import vsm as vsmapi
import json
from django.http import HttpResponse

LOG = logging.getLogger(__name__)

class IndexView(tables.DataTableView):
    table_class = ListPoolTable
    template_name = 'vsm/poolsmanagement/index.html'

    def get_data(self):
        pools = []

        try:
            rsp, body = vsmapi.pools_list(self.request, all_pools=True)
            if body:
                pools = body["pool"]
            logging.debug("resp body in view: %s" % pools)
        except:
            exceptions.handle(self.request,
                              _('Unable to retrieve storage pool list. '))
        pools = sorted(pools, lambda x,y: cmp(x['poolId'], y['poolId']))
        #pools = [pool for pool in pools if pool['tag'] != "SYSTEM"]
        return pools

class CreateView(TemplateView):
    template_name = 'vsm/poolsmanagement/create_replicated_pool.html'
    success_url = reverse_lazy('horizon:vsm:poolsmanagement:index')
    def get_context_data(self, **kwargs):
        context = super(CreateView, self).get_context_data(**kwargs)
        context["sg_list"] = []
        return context

class CreateErasureCodedPoolView(forms.ModalFormView):
    form_class = CreateErasureCodedPool
    template_name = 'vsm/poolsmanagement/create_erasure_coded_pool.html'
    success_url = reverse_lazy('horizon:vsm:poolsmanagement:index')

class AddCacheTierView(TemplateView):
    template_name = 'vsm/poolsmanagement/add_cache_tier.html'
    success_url = reverse_lazy('horizon:vsm:poolsmanagement:index')
    def get_context_data(self, **kwargs):
        context = super(AddCacheTierView, self).get_context_data(**kwargs)
        #get pool list
        pools = vsmapi.pool_status(None)
        context["pool_list"] = [(pool.pool_id, pool.name) for pool in pools if not pool.cache_tier_status]
        context["cache_mode_list"] = [('writeback', "Writeback"), ('readonly', "Read-only")]
        context["hit_set_type_list"] = [('bloom', "bloom")]
        #get the settings
        setting_list = vsmapi.get_settings(None)
        context["settings"] = dict([(setting.name, setting.value) for setting in setting_list])
        return context


class RemoveCacheTierView(forms.ModalFormView):
    form_class = RemoveCacheTier
    template_name = 'vsm/poolsmanagement/remove_cache_tier.html'
    success_url = reverse_lazy('horizon:vsm:poolsmanagement:index')


def add_cache_tier(request):
    status = ""
    msg = ""
    body = json.loads(request.body)

    try:
        ret = vsmapi.add_cache_tier(request,body=body)
        status = "OK"
        msg = "Add Cache Tier Successfully!"
    except:
        status = "Failed"
        msg = "Add Cache Tier Failed!"

    resp = dict(message=msg, status=status)
    resp = json.dumps(resp)
    return HttpResponse(resp)


def remove_cache_tier(request):
    status = ""
    msg = ""
    body = json.loads(request.body)
    try:
        ret = vsmapi.remove_cache_tier(request,body=body)
        status = "OK"
        msg = "Remove Cache Tier Successfully!"
    except:
        status = "Failed"
        msg = "Remove Cache Tier Failed!"

    resp = dict(message=msg, status=status)
    resp = json.dumps(resp)
    return HttpResponse(resp)


def create_replicated_pool(request):
    status = ""
    msg = ""
    body = json.loads(request.body)
    print body
    try:
        rsp, ret = vsmapi.create_storage_pool(request,body=body)
        res = str(ret['message']).strip()
        if res.startswith('pool') and res.endswith('created'):
            status = "OK"
            msg = "Created storage pool successfully!"
    except:
        status = "Failed"
        msg = "Create Replication Pool Failed!"
    resp = dict(message=msg, status=status)
    resp = json.dumps(resp)
    return HttpResponse(resp)

def create_ec_pool(request):
    status = ""
    msg = ""
    body = json.loads(request.body)
    print body

    try:
        rsp, ret = vsmapi.create_storage_pool(request,body=body)
        res = str(ret['message']).strip()
        if res.startswith('pool') and res.endswith('created'):
            status = "OK"
            msg = "Created storage pool successfully!"
    except:
        status = "Failed"
        msg = "Remove Cache Tier Failed!"

    resp = dict(message=msg, status=status)
    resp = json.dumps(resp)
    return HttpResponse(resp)

def get_default_pg_number_storage_group(request):
    storage_group_list = []
    rsp, group_list= vsmapi.get_storage_group_list(request)
    for key in group_list:
        rsp, default_pg_num = vsmapi.get_default_pg_num_by_storage_group(request, \
                                                   {'storage_group_name':group_list[key]})
        if default_pg_num['pg_num_default'] > 0:
            storage_group_list.append((key, group_list[key], default_pg_num['pg_num_default']))
    resp = json.dumps({"storage_group_list":storage_group_list})
    return HttpResponse(resp)



# Copyright 2014 Intel Corporation, All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.


from django.utils.translation import ugettext_lazy as _
from horizon import forms


import logging


LOG = logging.getLogger(__name__)


class CephUpgrade(forms.SelfHandlingForm):
    package_url = forms.CharField(label=_("Package URL"),
                            max_length=255,
                            min_length=1,
                            initial='http://ceph.com/debian-hammer/',
                            error_messages={
                            'required': _('This field is required.'),},
                            )

    key_url  = forms.CharField(label=_("Key URL"),
                            max_length=255,
                            min_length=1,
                            initial='https://ceph.com/git/?p=ceph.git;a=blob_plain;f=keys/release.asc',
                            error_messages={
                            'required': _('This field is required.'),},
                            )
    proxy  = forms.CharField(label=_("Proxy URL"),
                            max_length=255,
                            min_length=1,
                            required=False,
                            )
    ssh_user  = forms.CharField(label=_("SSH User Name"),
                            max_length=255,
                            min_length=1,
                            required=False,
                            )

    def __init__(self, request, *args, **kwargs):
        super(CephUpgrade, self).__init__(request, *args, **kwargs)


    def handle(self, request, data):
        pass



# Copyright 2014 Intel Corporation, All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

from django.core import validators
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages
from horizon.utils.validators import validate_port_range
from horizon.utils import fields
import logging

from vsm_dashboard.api import vsm as vsm_api

LOG = logging.getLogger(__name__)

class AddHost(forms.SelfHandlingForm):

    failure_url = 'horizon:vsm:storageservermgmt:index'
    host_name = forms.CharField(label=_("Host name"),
                            max_length=255,
                            min_length=1,
                            error_messages={
                            'required': _('This field is required.'),
                            'invalid': _("The string may only contain"
                                         " ASCII characters and numbers.")},
                            validators=[validators.validate_slug])

    password = forms.CharField(label=_("Password"),
                            widget=forms.PasswordInput(),
                            max_length=255,
                            min_length=1,
                            error_messages={
                            'required': _('This field is required.'),
                            'invalid': _("The string may only contain"
                                         " ASCII characters and numbers.")},
                            validators=[validators.validate_slug])

    server_type = forms.ChoiceField(label=_('Server Type'))
    zone = forms.ChoiceField(label=_('Zone'))

    def __init__(self, request, *args, **kwargs):
        super(AddHost, self).__init__(request, *args, **kwargs)

        self.fields['server_type'].choices = [('storage', 'storage'),
                                              ('monitor', 'monitor'),
                                              ('mixed', 'storage, monitor')]
        self.fields['zone'].choices = [('zone_a', 'zone_a'),
                                              ('zone_b', 'zone_b')]

    def handle(self, request, data):
        try:

            body = {
                    'pool': {
                        'name': data['name'],
                        'storageGroupId': data['storage_group'],
                        'replicationFactor': data['replication_factor'],
                        'clusterId': '0',
                        'createdBy': 'VSM'
                    }
            }
            rsp, ret = vsm_api.create_storage_pool(request,body=body)

            res = str(ret['message']).strip( )
            if res.startswith('pool') and res.endswith('created'):
                messages.success(request,
                                _('Successfully created storage pool: %s')
                                % data['name'])
            else:
                messages.error(request,
                                _('Because %s, failed to create storage pool')
                                % ret['message'])

            return ret
        except:
            redirect = reverse("horizon:vsm:poolsmanagement:index")
            exceptions.handle(request,
                              _('Unable to create storage pool.'),
                              redirect=redirect)

class AddHosts(forms.SelfHandlingForm):

    failure_url = 'horizon:vsm:storageservermgmt:index'
    host_name = forms.CharField(label=_("Host name"),
                            max_length=255,
                            min_length=1,
                            error_messages={
                            'required': _('This field is required.'),
                            'invalid': _("The string may only contain"
                                         " ASCII characters and numbers.")},
                            validators=[validators.validate_slug])

    password = forms.CharField(label=_("Password"),
                            widget=forms.PasswordInput(),
                            max_length=255,
                            min_length=1,
                            error_messages={
                            'required': _('This field is required.'),
                            'invalid': _("The string may only contain"
                                         " ASCII characters and numbers.")},
                            validators=[validators.validate_slug])

    server_type = forms.ChoiceField(label=_('Server Type'))
    zone = forms.ChoiceField(label=_('Zone'))

    def __init__(self, request, *args, **kwargs):
        super(AddHosts, self).__init__(request, *args, **kwargs)

        self.fields['server_type'].choices = [('storage', 'storage'),
                                              ('monitor', 'monitor'),
                                              ('mixed', 'storage, monitor')]
        self.fields['zone'].choices = [('zone_a', 'zone_a'),
                                              ('zone_b', 'zone_b')]

    def handle(self, request, data):
        try:

            body = {
                    'pool': {
                        'name': data['name'],
                        'storageGroupId': data['storage_group'],
                        'replicationFactor': data['replication_factor'],
                        'clusterId': '0',
                        'createdBy': 'VSM'
                    }
            }
            rsp, ret = vsm_api.create_storage_pool(request,body=body)

            res = str(ret['message']).strip( )
            if res.startswith('pool') and res.endswith('created'):
                messages.success(request,
                                _('Successfully created storage pool: %s')
                                % data['name'])
            else:
                messages.error(request,
                                _('Because %s, failed to create storage pool')
                                % ret['message'])

            return ret
        except:
            redirect = reverse("horizon:vsm:poolsmanagement:index")
            exceptions.handle(request,
                              _('Unable to create storage pool.'),
                              redirect=redirect)





# Copyright 2014 Intel Corporation, All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

from django import template
from django.views.generic import View
import logging
LOG = logging.getLogger(__name__)

class SummaryRenderer(View):
    def __init__(self, request):
        self.request = request

    def render(self):
        summary_template = template.loader.get_template("vsm/summary.html")

        try:
            datas = self.get_summary()
        except Exception,e:
            datas = {}
            #LOG.info("CRITICAL: %s"%e)
            #LOG.error(e)

        try:
            summary_list = self.get_summary_list()
        except Exception,e:
            summary_list = []

        context = template.Context({"name": self.name,
                                    "verbose_name": self.verbose_name,
                                    "detail": getattr(self, "detail", None),
                                    "datas": datas,
                                    "summary_list": summary_list})
        return summary_template.render(context)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Corporation, All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django import shortcuts
from django.views.decorators import vary
from django.http import HttpResponse
from django.http import HttpResponseRedirect
from django.template.loader import get_template
from django.template import Context
from django.conf import settings

import horizon

from openstack_auth.views import login

from vsm_dashboard import api

def get_user_home(user):
    #if user.is_superuser:
    #    return horizon.get_dashboard('admin').get_absolute_url()
    return horizon.get_dashboard('vsm').get_absolute_url()

def license_gate(request):
    _template = get_template('license.html')
    context = Context()
    html = _template.render(context=context)
    return HttpResponse(html)

def license_accept(request):
    api.vsm.license_create(request)
    api.vsm.license_update(request, True)
	#Redirect to cluster create page if user accept license.
	#TODO(fengqian): When trun to cluster create page,
	#All other functions should be unavaliable.
    return shortcuts.redirect('/dashboard/vsm/clustermgmt')

def license_cancel(request):
    return HttpResponseRedirect(settings.LOGOUT_URL)

@vary.vary_on_cookie
def splash(request):
    if request.user.is_authenticated():
        #license_status = api.vsm.license_get(request)[1]
        #if license_status is None or\
        #    license_status.get('license_accept', False) == False:
        #        return license_gate(request)
        #else:
        return shortcuts.redirect(get_user_home(request.user))
    form = login(request)
    request.session.clear()
    request.session.set_test_cookie()
    return shortcuts.render(request, 'splash.html', {'form': form})




# Copyright 2014 Intel Corporation, All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

json = """[
    {
        "OS-EXT-STS:vm_state": "active",
        "OS-EXT-SRV-ATTR:instance_name": "instance-00000001",
        "flavor": {
            "name": "m1.tiny",
            "ram": 512,
            "vcpus": 1,
            "swap": "",
            "rxtx_factor": 1,
            "OS-FLV-EXT-DATA:ephemeral": 0,
            "disk": 0,
            "id": "1"
        },
        "user": {
            "id": "4e396e83369f4aab81a08bf93b5cb7b3",
            "name": "admin",
            "email": "admin@example.com"
        },
        "tenant": {
            "id": "cd82e5ce8e274f568b7b949e3cf6b350",
            "name": "demo"
        },
        "OS-EXT-STS:task_state": null,
        "image": {
            "id": "fd9bbb0a-2ebf-405f-8773-55574751388a"
        },
        "id": "242c671a-a9f8-4a38-90b6-f196649f520d",
        "user_id": "4e396e83369f4aab81a08bf93b5cb7b3",
        "status": "ACTIVE",
        "updated": "2012-03-22T20:59:38Z",
        "hostId": "0165abf99d3e318b26eef9e0465671c6e13e40ae13d3bfe2625105cf",
        "OS-EXT-SRV-ATTR:host": "ubuntu",
        "name": "instanceOne",
        "created": 1332478770,
        "tenant_id": "cd82e5ce8e274f568b7b949e3cf6b350"
    },
    {
        "OS-EXT-STS:vm_state": "active",
        "OS-EXT-SRV-ATTR:instance_name": "instance-00000001",
        "flavor": {
            "name": "m2.small",
            "ram": 1024,
            "vcpus": 2,
            "swap": "",
            "rxtx_factor": 1,
            "OS-FLV-EXT-DATA:ephemeral": 0,
            "disk": 0,
            "id": "2"
        },
        "user": {
            "id": "4e396e8g369f4aab81a08bf93b5cb7b3",
            "name": "demo"
        },
        "tenant": {
            "id": "cd82e5ce8e274f568b7b949e3cf6b350",
            "name": "demo"
        },
        "OS-EXT-STS:task_state": null,
        "image": {
            "id": "fd9bbb0a-2ebf-405f-8773-55574751388a"
        },
        "id": "242c671a-a9f8-4a38-90b6-f196649f520d",
        "user_id": "4e396e83369f4aab81a08bf93b5cb7b3",
        "status": "ACTIVE",
        "updated": "2012-03-22T20:59:38Z",
        "hostId": "0165abf99d3e318b26eef9e0465671c6e13e40ae13d3bfe2625105cf",
        "OS-EXT-SRV-ATTR:host": "other",
        "name": "instanceOne",
        "created": 1332478770,
        "tenant_id": "cd82e5ce8e274f568b7b949e3cf6b350"
    },
    {
        "OS-EXT-STS:vm_state": "active",
        "OS-EXT-SRV-ATTR:instance_name": "instance-00000001",
        "flavor": {
            "name": "m1.tiny",
            "ram": 512,
            "vcpus": 1,
            "swap": "",
            "rxtx_factor": 1,
            "OS-FLV-EXT-DATA:ephemeral": 0,
            "disk": 0,
            "id": "1"
        },
        "user": {
            "id": "4e396e8336gf4aab81a08bf93b5cb7b3",
            "name": "demo",
            "email": "admin@example.com"
        },
        "tenant": {
            "id": "dd82e5ce8e274f568b7b949e3cf6b350",
            "name": "other"
        },
        "OS-EXT-STS:task_state": null,
        "image": {
            "id": "fd9bbb0a-2ebf-405f-8773-55574751388a"
        },
        "id": "242c671a-a9f8-4a38-90b6-f196649f520d",
        "user_id": "4e396e83369f4aab81a08bf93b5cb7b3",
        "status": "ACTIVE",
        "updated": "2012-03-22T20:59:38Z",
        "hostId": "0165abf99d3e318b26eef9e0465671c6e13e40ae13d3bfe2625105cf",
        "OS-EXT-SRV-ATTR:host": "ubuntu",
        "name": "instanceOne",
        "created": 1332478770,
        "tenant_id": "cd82e5ce8e274f568b7b949e3cf6b350"
    },
    {
        "OS-EXT-STS:vm_state": "active",
        "OS-EXT-SRV-ATTR:instance_name": "instance-00000001",
        "flavor": {
            "name": "m2.small",
            "ram": 1024,
            "vcpus": 2,
            "swap": "",
            "rxtx_factor": 1,
            "OS-FLV-EXT-DATA:ephemeral": 0,
            "disk": 0,
            "id": "2"
        },
        "user": {
            "id": "4e396e83369f4aab81a08bf93b5cb7b3",
            "name": "admin",
            "email": "admin@example.com"
        },
        "tenant": {
            "id": "cd82e5ce8e274f568b7b949e3cf6b350",
            "name": "demo"
        },
        "OS-EXT-STS:task_state": null,
        "image": {
            "id": "fd9bbb0a-2ebf-405f-8773-55574751388a"
        },
        "id": "242c671a-a9f8-4a38-90b6-f196649f520d",
        "user_id": "4e396e83369f4aab81a08bf93b5cb7b3",
        "status": "ACTIVE",
        "updated": "2012-03-22T20:59:38Z",
        "hostId": "0165abf99d3e318b26eef9e0465671c6e13e40ae13d3bfe2625105cf",
        "OS-EXT-SRV-ATTR:host": "ubuntu",
        "name": "instanceOne",
        "created": 1332478770,
        "tenant_id": "cd82e5ce8e274f568b7b949e3cf6b350"
    },
    {
        "OS-EXT-STS:vm_state": "active",
        "OS-EXT-SRV-ATTR:instance_name": "instance-00000001",
        "flavor": {
            "name": "m1.tiny",
            "ram": 512,
            "vcpus": 1,
            "swap": "",
            "rxtx_factor": 1,
            "OS-FLV-EXT-DATA:ephemeral": 0,
            "disk": 0,
            "id": "1"
        },
        "user": {
            "id": "4e396e83369f4aab81a08bf93b5cb7b3",
            "name": "admin",
            "email": "admin@example.com"
        },
        "tenant": {
            "id": "dd82e5ce8e274f568b7b949e3cf6b350",
            "name": "other"
        },
        "OS-EXT-STS:task_state": null,
        "image": {
            "id": "fd9bbb0a-2ebf-405f-8773-55574751388a"
        },
        "id": "242c671a-a9f8-4a38-90b6-f196649f520d",
        "user_id": "4e396e83369f4aab81a08bf93b5cb7b3",
        "status": "ACTIVE",
        "updated": "2012-03-22T20:59:38Z",
        "hostId": "0165abf99d3e318b26eef9e0465671c6e13e40ae13d3bfe2625105cf",
        "OS-EXT-SRV-ATTR:host": "ubuntu",
        "name": "instanceOne",
        "created": 1332478770,
        "tenant_id": "cd82e5ce8e274f568b7b949e3cf6b350"
    },
    {
        "OS-EXT-STS:vm_state": "active",
        "OS-EXT-SRV-ATTR:instance_name": "instance-00000001",
        "flavor": {
            "name": "m3.medium",
            "ram": 1024,
            "vcpus": 4,
            "swap": "",
            "rxtx_factor": 1,
            "OS-FLV-EXT-DATA:ephemeral": 0,
            "disk": 0,
            "id": "1"
        },
        "user": {
            "id": "4e396e83369f4aab81a08bf93b5cb7b3",
            "name": "admin",
            "email": "admin@example.com"
        },
        "tenant": {
            "id": "dd82e5ce8e274f568b7b949e3cf6b350",
            "name": "other"
        },
        "OS-EXT-STS:task_state": null,
        "image": {
            "id": "fd9bbb0a-2ebf-405f-8773-55574751388a"
        },
        "id": "242c671a-a9f8-4a38-90b6-f196649f520d",
        "user_id": "4e396e83369f4aab81a08bf93b5cb7b3",
        "status": "ACTIVE",
        "updated": "2012-03-22T20:59:38Z",
        "hostId": "0165abf99d3e318b26eef9e0465671c6e13e40ae13d3bfe2625105cf",
        "OS-EXT-SRV-ATTR:host": "other",
        "name": "instanceOne",
        "created": 1332478770,
        "tenant_id": "cd82e5ce8e274f568b7b949e3cf6b350"
    }
]
"""



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2012 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2012 Nebula, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from collections import Sequence
import logging

from django.conf import settings

from horizon import exceptions

__all__ = ('APIResourceWrapper', 'APIDictWrapper',
           'get_service_from_catalog', 'url_for',)

LOG = logging.getLogger(__name__)

class APIResourceWrapper(object):
    """ Simple wrapper for api objects

        Define _attrs on the child class and pass in the
        api object as the only argument to the constructor
    """
    _attrs = []

    def __init__(self, apiresource):
        self._apiresource = apiresource

    def __getattr__(self, attr):
        if attr in self._attrs:
            # __getattr__ won't find properties
            return self._apiresource.__getattribute__(attr)
        else:
            msg = ('Attempted to access unknown attribute "%s" on '
                   'APIResource object of type "%s" wrapping resource of '
                   'type "%s".') % (attr, self.__class__,
                                    self._apiresource.__class__)
            LOG.debug(exceptions.error_color(msg))
            raise AttributeError(attr)

class APIDictWrapper(object):
    """ Simple wrapper for api dictionaries

        Some api calls return dictionaries.  This class provides identical
        behavior as APIResourceWrapper, except that it will also behave as a
        dictionary, in addition to attribute accesses.

        Attribute access is the preferred method of access, to be
        consistent with api resource objects from novclient.
    """
    def __init__(self, apidict):
        self._apidict = apidict

    def __getattr__(self, attr):
        try:
            return self._apidict[attr]
        except KeyError:
            msg = 'Unknown attribute "%(attr)s" on APIResource object ' \
                  'of type "%(cls)s"' % {'attr': attr, 'cls': self.__class__}
            LOG.debug(exceptions.error_color(msg))
            raise AttributeError(msg)

    def __getitem__(self, item):
        try:
            return self.__getattr__(item)
        except AttributeError, e:
            # caller is expecting a KeyError
            raise KeyError(e)

    def get(self, item, default=None):
        try:
            return self.__getattr__(item)
        except AttributeError:
            return default

class Quota(object):
    """Wrapper for individual limits in a quota."""
    def __init__(self, name, limit):
        self.name = name
        self.limit = limit

    def __repr__(self):
        return "<Quota: (%s, %s)>" % (self.name, self.limit)

class QuotaSet(Sequence):
    """
    Wrapper for client QuotaSet objects which turns the individual quotas
    into Quota objects for easier handling/iteration.

    `QuotaSet` objects support a mix of `list` and `dict` methods; you can use
    the bracket notiation (`qs["my_quota"] = 0`) to add new quota values, and
    use the `get` method to retrieve a specific quota, but otherwise it
    behaves much like a list or tuple, particularly in supporting iteration.
    """
    def __init__(self, apiresource=None):
        self.items = []
        if apiresource:
            for k, v in apiresource._info.items():
                if k == 'id':
                    continue
                self[k] = v

    def __setitem__(self, k, v):
            v = int(v) if v is not None else v
            q = Quota(k, v)
            self.items.append(q)

    def __getitem__(self, index):
        return self.items[index]

    def __len__(self):
        return len(self.items)

    def __repr__(self):
        return repr(self.items)

    def get(self, key, default=None):
        match = [quota for quota in self.items if quota.name == key]
        return match.pop() if len(match) else Quota(key, default)

def get_service_from_catalog(catalog, service_type):
    if catalog:
        for service in catalog:
            if service['type'] == service_type:
                return service
    return None

def url_for(request, service_type, admin=False, endpoint_type=None):
    endpoint_type = endpoint_type or getattr(settings,
                                             'OPENSTACK_ENDPOINT_TYPE',
                                             'publicURL')
    catalog = request.user.service_catalog
    service = get_service_from_catalog(catalog, service_type)
    if service:
        try:
            if admin:
                return service['endpoints'][0]['adminURL']
            else:
                return service['endpoints'][0][endpoint_type]
        except (IndexError, KeyError):
            raise exceptions.ServiceCatalogException(service_type)
    else:
        raise exceptions.ServiceCatalogException(service_type)

def is_service_enabled(request, service_type, service_name=None):
    service = get_service_from_catalog(request.user.service_catalog,
                                       service_type)
    if service and service_name:
        return service['name'] == service_name
    else:
        return service is not None




# Copyright 2014 Intel Corporation, All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import time

from django.utils.dateparse import parse_datetime
from django.utils.datastructures import SortedDict
from django.utils.translation import ugettext_lazy as _

from vsm_dashboard import api

from horizon import exceptions

from .test_data import json as test_json

def get_admin_tenant(request):
    # TODO
    tenants = api.keystone.tenant_list(request)
    tenant_dict = SortedDict([(t.name, t) for t in tenants])
    admin_tenant = tenant_dict.get("admin", None)
    return admin_tenant

def correlate_tenants(request, instances):
    # Gather our tenants to correlate against IDs
    try:
        tenants = api.keystone.tenant_list(request, admin=True)
    except:
        tenants = []
        msg = _('Unable to retrieve instance tenant information.')
        exceptions.handle(request, msg)
    tenant_dict = SortedDict([(t.id, t) for t in tenants])
    for inst in instances:
        tenant = tenant_dict.get(inst.tenant_id, None)
        inst._apiresource._info['tenant'] = tenant._info
        inst.tenant = tenant

def correlate_flavors(request, instances):
    # Gather our flavors to correlate against IDs
    try:
        flavors = api.nova.flavor_list(request)
    except:
        flavors = []
        msg = _('Unable to retrieve instance size information.')
        exceptions.handle(request, msg)

    flavors_dict = SortedDict([(f.id, f) for f in flavors])
    for inst in instances:
        flavor = flavors_dict.get(inst.flavor["id"], None)
        inst._apiresource._info['flavor'] = flavor._info
        inst.flavor = flavor

def correlate_users(request, instances):
    # Gather our users to correlate against IDs
    try:
        users = api.keystone.user_list(request)
    except:
        users = []
        msg = _('Unable to retrieve instance user information.')
        exceptions.handle(request, msg)
    user_dict = SortedDict([(u.id, u) for u in users])
    for inst in instances:
        user = user_dict.get(inst.user_id, None)
        inst._apiresource._info['user'] = user._info
        inst.user = user

def calculate_ages(instances):
    for instance in instances:
        dt = parse_datetime(instance._apiresource.created)
        timestamp = time.mktime(dt.timetuple())
        instance._apiresource._info['created'] = timestamp
        instance.age = dt

def get_fake_instances_data(request):
    import json
    from novaclient.v1_1.servers import Server, ServerManager
    from horizon.api.nova import Server as HServer
    instances = [HServer(Server(ServerManager(None), i), request)
                 for i in json.loads(test_json)]
    for i in instances:
        i._loaded = True
    instances += instances
    return instances

def get_instances_data(request):
    instances = api.nova.server_list(request, all_tenants=True)

    # Get the useful data... thanks Nova :-P
    if instances:
        correlate_flavors(request, instances)
        correlate_tenants(request, instances)
        correlate_users(request, instances)
        calculate_ages(instances)

    return instances

def checkbox_transform(text):
    return """<input type="checkbox" name="checkbox1" value="checkbox"> text """



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2012 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2012 Nebula, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from functools import wraps
import os

from django import http
from django.conf import settings
from django.contrib.messages.storage import default_storage
from django.contrib.auth.middleware import AuthenticationMiddleware
from django.core.handlers import wsgi
from django.test.client import RequestFactory
from django.utils import unittest

import glanceclient
from keystoneclient.v2_0 import client as keystone_client
from novaclient.v1_1 import client as nova_client
from quantumclient.v2_0 import client as quantum_client
from swiftclient import client as swift_client
from vsmclient import client as vsm_client

import httplib2
import mox

from openstack_auth import utils, user

from horizon.test import helpers as horizon_helpers
from horizon import middleware

from vsm_dashboard import api
from vsm_dashboard import context_processors
from vsm_dashboard.test.test_data.utils import load_test_data

# Makes output of failing mox tests much easier to read.
wsgi.WSGIRequest.__repr__ = lambda self: "<class 'django.http.HttpRequest'>"

def create_stubs(stubs_to_create={}):
    if not isinstance(stubs_to_create, dict):
        raise TypeError("create_stub must be passed a dict, but a %s was "
                        "given." % type(stubs_to_create).__name__)

    def inner_stub_out(fn):
        @wraps(fn)
        def instance_stub_out(self):
            for key in stubs_to_create:
                if not (isinstance(stubs_to_create[key], tuple) or
                        isinstance(stubs_to_create[key], list)):
                    raise TypeError("The values of the create_stub "
                                    "dict must be lists or tuples, but "
                                    "is a %s."
                                    % type(stubs_to_create[key]).__name__)

                for value in stubs_to_create[key]:
                    self.mox.StubOutWithMock(key, value)
            return fn(self)
        return instance_stub_out
    return inner_stub_out

class RequestFactoryWithMessages(RequestFactory):
    def get(self, *args, **kwargs):
        req = super(RequestFactoryWithMessages, self).get(*args, **kwargs)
        req.user = utils.get_user(req)
        req.session = []
        req._messages = default_storage(req)
        return req

    def post(self, *args, **kwargs):
        req = super(RequestFactoryWithMessages, self).post(*args, **kwargs)
        req.user = utils.get_user(req)
        req.session = []
        req._messages = default_storage(req)
        return req

@unittest.skipIf(os.environ.get('SKIP_UNITTESTS', False),
                     "The SKIP_UNITTESTS env variable is set.")
class TestCase(horizon_helpers.TestCase):
    """
    Specialized base test case class for Horizon which gives access to
    numerous additional features:

      * A full suite of test data through various attached objects and
        managers (e.g. ``self.servers``, ``self.user``, etc.). See the
        docs for :class:`~horizon.tests.test_data.utils.TestData` for more
        information.
      * The ``mox`` mocking framework via ``self.mox``.
      * A set of request context data via ``self.context``.
      * A ``RequestFactory`` class which supports Django's ``contrib.messages``
        framework via ``self.factory``.
      * A ready-to-go request object via ``self.request``.
      * The ability to override specific time data controls for easier testing.
      * Several handy additional assertion methods.
    """
    def setUp(self):
        load_test_data(self)
        self.mox = mox.Mox()
        self.factory = RequestFactoryWithMessages()
        self.context = {'authorized_tenants': self.tenants.list()}

        def fake_conn_request(*args, **kwargs):
            raise Exception("An external URI request tried to escape through "
                            "an httplib2 client. Args: %s, kwargs: %s"
                            % (args, kwargs))

        self._real_conn_request = httplib2.Http._conn_request
        httplib2.Http._conn_request = fake_conn_request

        self._real_context_processor = context_processors.openstack
        context_processors.openstack = lambda request: self.context

        self._real_get_user = utils.get_user
        tenants = self.context['authorized_tenants']
        self.setActiveUser(id=self.user.id,
                           token=self.token,
                           username=self.user.name,
                           tenant_id=self.tenant.id,
                           service_catalog=self.service_catalog,
                           authorized_tenants=tenants)
        self.request = http.HttpRequest()
        self.request.session = self.client._session()
        self.request.session['token'] = self.token.id
        middleware.HorizonMiddleware().process_request(self.request)
        AuthenticationMiddleware().process_request(self.request)
        os.environ["HORIZON_TEST_RUN"] = "True"

    def tearDown(self):
        self.mox.UnsetStubs()
        httplib2.Http._conn_request = self._real_conn_request
        context_processors.openstack = self._real_context_processor
        utils.get_user = self._real_get_user
        self.mox.VerifyAll()
        del os.environ["HORIZON_TEST_RUN"]

    def setActiveUser(self, id=None, token=None, username=None, tenant_id=None,
                        service_catalog=None, tenant_name=None, roles=None,
                        authorized_tenants=None, enabled=True):
        def get_user(request):
            return user.User(id=id,
                             token=token,
                             user=username,
                             tenant_id=tenant_id,
                             service_catalog=service_catalog,
                             roles=roles,
                             enabled=enabled,
                             authorized_tenants=authorized_tenants,
                             endpoint=settings.OPENSTACK_KEYSTONE_URL)
        utils.get_user = get_user

    def assertRedirectsNoFollow(self, response, expected_url):
        """
        Asserts that the given response issued a 302 redirect without
        processing the view which is redirected to.
        """
        assert (response.status_code / 100 == 3), \
            "The response did not return a redirect."
        self.assertEqual(response._headers.get('location', None),
                         ('Location', settings.TESTSERVER + expected_url))
        self.assertEqual(response.status_code, 302)

    def assertNoFormErrors(self, response, context_name="form"):
        """
        Asserts that the response either does not contain a form in it's
        context, or that if it does, that form has no errors.
        """
        context = getattr(response, "context", {})
        if not context or context_name not in context:
            return True
        errors = response.context[context_name]._errors
        assert len(errors) == 0, \
               "Unexpected errors were found on the form: %s" % errors

    def assertFormErrors(self, response, count=0, message=None,
                         context_name="form"):
        """
        Asserts that the response does contain a form in it's
        context, and that form has errors, if count were given,
        it must match the exact numbers of errors
        """
        context = getattr(response, "context", {})
        assert (context and context_name in context), \
            "The response did not contain a form."
        errors = response.context[context_name]._errors
        if count:
            assert len(errors) == count, \
               "%d errors were found on the form, %d expected" % \
               (len(errors), count)
            if message and message not in unicode(errors):
                self.fail("Expected message not found, instead found: %s"
                          % ["%s: %s" % (key, [e for e in field_errors]) for
                             (key, field_errors) in errors.items()])
        else:
            assert len(errors) > 0, "No errors were found on the form"

class BaseAdminViewTests(TestCase):
    """
    A ``TestCase`` subclass which sets an active user with the "admin" role
    for testing admin-only views and functionality.
    """
    def setActiveUser(self, *args, **kwargs):
        if "roles" not in kwargs:
            kwargs['roles'] = [self.roles.admin._info]
        super(BaseAdminViewTests, self).setActiveUser(*args, **kwargs)

class APITestCase(TestCase):
    """
    The ``APITestCase`` class is for use with tests which deal with the
    underlying clients rather than stubbing out the
    vsm_dashboard.api.* methods.
    """
    def setUp(self):
        super(APITestCase, self).setUp()

        def fake_keystoneclient(request, admin=False):
            """
            Wrapper function which returns the stub keystoneclient. Only
            necessary because the function takes too many arguments to
            conveniently be a lambda.
            """
            return self.stub_keystoneclient()

        # Store the original clients
        self._original_glanceclient = api.glance.glanceclient
        self._original_keystoneclient = api.keystone.keystoneclient
        self._original_novaclient = api.nova.novaclient
        self._original_quantumclient = api.quantum.quantumclient
        self._original_vsmclient = api.vsm.vsmclient

        # Replace the clients with our stubs.
        api.glance.glanceclient = lambda request: self.stub_glanceclient()
        api.keystone.keystoneclient = fake_keystoneclient
        api.nova.novaclient = lambda request: self.stub_novaclient()
        api.quantum.quantumclient = lambda request: self.stub_quantumclient()
        api.vsm.vsmclient = lambda request: self.stub_vsmclient()

    def tearDown(self):
        super(APITestCase, self).tearDown()
        api.glance.glanceclient = self._original_glanceclient
        api.nova.novaclient = self._original_novaclient
        api.keystone.keystoneclient = self._original_keystoneclient
        api.quantum.quantumclient = self._original_quantumclient
        api.vsm.vsmclient = self._original_vsmclient

    def stub_novaclient(self):
        if not hasattr(self, "novaclient"):
            self.mox.StubOutWithMock(nova_client, 'Client')
            self.novaclient = self.mox.CreateMock(nova_client.Client)
        return self.novaclient

    def stub_vsmclient(self):
        if not hasattr(self, "vsmclient"):
            self.mox.StubOutWithMock(vsm_client, 'Client')
            self.vsmclient = self.mox.CreateMock(vsm_client.Client)
        return self.vsmclient

    def stub_keystoneclient(self):
        if not hasattr(self, "keystoneclient"):
            self.mox.StubOutWithMock(keystone_client, 'Client')
            # NOTE(saschpe): Mock the 'auth_token' property specifically,
            # MockObject.__init__ ignores properties altogether:
            keystone_client.Client.auth_token = 'foo'
            self.keystoneclient = self.mox.CreateMock(keystone_client.Client)
        return self.keystoneclient

    def stub_glanceclient(self):
        if not hasattr(self, "glanceclient"):
            self.mox.StubOutWithMock(glanceclient, 'Client')
            self.glanceclient = self.mox.CreateMock(glanceclient.Client)
        return self.glanceclient

    def stub_quantumclient(self):
        if not hasattr(self, "quantumclient"):
            self.mox.StubOutWithMock(quantum_client, 'Client')
            self.quantumclient = self.mox.CreateMock(quantum_client.Client)
        return self.quantumclient

    def stub_swiftclient(self, expected_calls=1):
        if not hasattr(self, "swiftclient"):
            self.mox.StubOutWithMock(swift_client, 'Connection')
            self.swiftclient = self.mox.CreateMock(swift_client.Connection)
            while expected_calls:
                swift_client.Connection(None,
                                        mox.IgnoreArg(),
                                        None,
                                        preauthtoken=mox.IgnoreArg(),
                                        preauthurl=mox.IgnoreArg(),
                                        auth_version="2.0") \
                            .AndReturn(self.swiftclient)
                expected_calls -= 1
        return self.swiftclient

@unittest.skipUnless(os.environ.get('WITH_SELENIUM', False),
                     "The WITH_SELENIUM env variable is not set.")
class SeleniumTestCase(horizon_helpers.SeleniumTestCase):

    def setUp(self):
        super(SeleniumTestCase, self).setUp()

        load_test_data(self)
        self.mox = mox.Mox()

        self._real_get_user = utils.get_user
        self.setActiveUser(id=self.user.id,
                           token=self.token,
                           username=self.user.name,
                           tenant_id=self.tenant.id,
                           service_catalog=self.service_catalog,
                           authorized_tenants=self.tenants.list())
        os.environ["HORIZON_TEST_RUN"] = "True"

    def tearDown(self):
        self.mox.UnsetStubs()
        utils.get_user = self._real_get_user
        self.mox.VerifyAll()
        del os.environ["HORIZON_TEST_RUN"]

    def setActiveUser(self, id=None, token=None, username=None, tenant_id=None,
                        service_catalog=None, tenant_name=None, roles=None,
                        authorized_tenants=None, enabled=True):
        def get_user(request):
            return user.User(id=id,
                             token=token,
                             user=username,
                             tenant_id=tenant_id,
                             service_catalog=service_catalog,
                             roles=roles,
                             enabled=enabled,
                             authorized_tenants=authorized_tenants,
                             endpoint=settings.OPENSTACK_KEYSTONE_URL)
        utils.get_user = get_user

class SeleniumAdminTestCase(SeleniumTestCase):
    """
    A ``TestCase`` subclass which sets an active user with the "admin" role
    for testing admin-only views and functionality.
    """
    def setActiveUser(self, *args, **kwargs):
        if "roles" not in kwargs:
            kwargs['roles'] = [self.roles.admin._info]
        super(SeleniumAdminTestCase, self).setActiveUser(*args, **kwargs)




# Copyright 2014 Intel Corporation, All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import time
import json
import urllib2

from django.utils.dateparse import parse_datetime
from django.utils.datastructures import SortedDict
from django.utils.translation import ugettext_lazy as _

from vsm_dashboard import api

from horizon import exceptions

from .test_data import json as test_json

def correlate_tenants(request, instances):
    # Gather our tenants to correlate against IDs
    try:
        tenants = api.keystone.tenant_list(request, admin=True)
    except:
        tenants = []
        msg = _('Unable to retrieve instance tenant information.')
        exceptions.handle(request, msg)
    tenant_dict = SortedDict([(t.id, t) for t in tenants])
    for inst in instances:
        tenant = tenant_dict.get(inst.tenant_id, None)
        inst._apiresource._info['tenant'] = tenant._info
        inst.tenant = tenant

def correlate_flavors(request, instances):
    # Gather our flavors to correlate against IDs
    try:
        flavors = api.nova.flavor_list(request)
    except:
        flavors = []
        msg = _('Unable to retrieve instance size information.')
        exceptions.handle(request, msg)

    flavors_dict = SortedDict([(f.id, f) for f in flavors])
    for inst in instances:
        flavor = flavors_dict.get(inst.flavor["id"], None)
        inst._apiresource._info['flavor'] = flavor._info
        inst.flavor = flavor

def correlate_users(request, instances):
    # Gather our users to correlate against IDs
    try:
        users = api.keystone.user_list(request)
    except:
        users = []
        msg = _('Unable to retrieve instance user information.')
        exceptions.handle(request, msg)
    user_dict = SortedDict([(u.id, u) for u in users])
    for inst in instances:
        user = user_dict.get(inst.user_id, None)
        inst._apiresource._info['user'] = user._info
        inst.user = user

def calculate_ages(instances):
    for instance in instances:
        dt = parse_datetime(instance._apiresource.created)
        timestamp = time.mktime(dt.timetuple())
        instance._apiresource._info['created'] = timestamp
        instance.age = dt

def get_fake_instances_data(request):
    import json
    from novaclient.v1_1.servers import Server, ServerManager
    from horizon.api.nova import Server as HServer
    instances = [HServer(Server(ServerManager(None), i), request)
                 for i in json.loads(test_json)]
    for i in instances:
        i._loaded = True
    instances += instances
    return instances

def get_instances_data(request):
    instances = api.nova.server_list(request, all_tenants=True)

    # Get the useful data... thanks Nova :-P
    if instances:
        correlate_flavors(request, instances)
        correlate_tenants(request, instances)
        correlate_users(request, instances)
        calculate_ages(instances)

    return instances


def list_cinder_service(host, token, tenant_id):
    req_url = "http://%s:8776/v1/%s" % (host, tenant_id) + "/os-services"
    req = urllib2.Request(req_url)
    req.get_method = lambda: 'GET'
    req.add_header("content-type", "application/json")
    req.add_header("X-Auth-Token", token)
    resp = urllib2.urlopen(req)
    cinder_service_dict = json.loads(resp.read())
    cinder_service_list = cinder_service_dict['services']
    return cinder_service_list

def from_keystone_v3(tenant_name, username, password, auth_url, region_name):
    auth_url = auth_url + "/auth/tokens"
    user_id = username
    user_password = password
    project_id = tenant_name
    auth_data = {
        "auth": {
            "identity": {
                "methods": ["password"],
                "password": {
                    "user": {
                        "id": user_id,
                        "password": user_password
                    }
                }
            },
            "scope": {
                "project": {
                    "id": project_id
                }
            }
        }
    }
    auth_request = urllib2.Request(auth_url)
    auth_request.add_header("content-type", "application/json")
    auth_request.add_header('Accept', 'application/json')
    auth_request.add_header('User-Agent', 'python-mikeyp')
    auth_request.add_data(json.dumps(auth_data))
    auth_response = urllib2.urlopen(auth_request)
    response_data = json.loads(auth_response.read())
    services_list = response_data['token']['catalog']
    endpoints_list = []
    _url = None
    for service in services_list:
        service_type = service['type']
        service_name = service['name']
        if service_type == "volume" and service_name == "cinder":
            endpoints_list = service['endpoints']
            break
    for endpoint in endpoints_list:
        interface = endpoint['interface']
        region_id = endpoint['region_id']
        if region_name:
            if interface == "public" and region_id == region_name:
                _url = endpoint['url']
                break
        else:
            if len(endpoints_list) == 3:
                _url = endpoint['url']
                break
    host = _url.split('/')[2].split(':')[0]
    _token = auth_response.info().getheader('X-Subject-Token')
    return _token, host



def from_keystone_v2(tenant_name, username, password, auth_url, region_name):
    auth_url = auth_url + "/tokens"
    auth_data = {
        "auth": {
            "tenantName": tenant_name,
            "passwordCredentials":{
                "username": username,
                "password": password
            }
        }
    }

    auth_request = urllib2.Request(auth_url)
    auth_request.add_header("content-type", "application/json")
    auth_request.add_header('Accept', 'application/json')
    auth_request.add_header('User-Agent', 'python-mikeyp')
    auth_request.add_data(json.dumps(auth_data))
    auth_response = urllib2.urlopen(auth_request)
    response_data = json.loads(auth_response.read())

    _token = response_data['access']['token']['id']

    service_list = response_data['access']['serviceCatalog']
    _url = None
    for service in service_list:
        if service['type'] == 'volume' and service['name'] == 'cinder':
            for endpoint in service['endpoints']:
                if region_name != None and endpoint['region'] == region_name:
                    _url = endpoint['publicURL']
                    break
                elif region_name == None or region_name == "" and len(service['endpoints']) == 1:
                    _url = endpoint['publicURL']
                    break

    url_id = _url.split('/')[-1]
    host = _url.split('/')[2].split(':')[0]
    return _token, url_id, host



# Copyright 2014 Intel Corporation, All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

from django.conf.urls import patterns, url
from .views import IndexView
from .views import DevicesAction
from .views import add_new_osd
from .views import add_new_osd_action,check_device_path,get_smart_info,get_available_disks
from .views import restart_osd,remove_osd,restore_osd

urlpatterns = patterns('',
    url(r'^$', IndexView.as_view(), name='index'),
    url(r'^add_new_osd/$', add_new_osd, name='add_new_osd'),
    url(r'^add_new_osd_action/$', add_new_osd_action, name='add_new_osd_action'),
    url(r'^check_device_path/$', check_device_path, name='check_device_path'),
    url(r'^get_available_disks/$', get_available_disks, name='get_available_disks'),
    url(r'^devices/(?P<action>\w+)$', DevicesAction, name='devicesaction'),
    url(r'^get_smart_info/$', get_smart_info, name='get_smart_info'),

    url(r'^restart_osd/$', restart_osd, name='restart_osd'),
    url(r'^remove_osd/$', remove_osd, name='remove_osd'),
    url(r'^restore_osd/$', restore_osd, name='restore_osd'),

    url(r'/', IndexView.as_view(), name='index'),
)




# vim: tabstop=4 shiftwidth=4 softtabstop=4

#
# Copyright 2014 Intel Corporation, All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
import os
import sys
import warnings

from vsm_dashboard import exceptions

warnings.formatwarning = lambda message, category, *args, **kwargs: \
                                '%s: %s' % (category.__name__, message)

ROOT_PATH = os.path.dirname(os.path.abspath(__file__))
BIN_DIR = '/usr/bin'

if ROOT_PATH not in sys.path:
    sys.path.append(ROOT_PATH)

DEBUG = False
TEMPLATE_DEBUG = DEBUG

SECRET_KEY = None

if not SECRET_KEY:
    from horizon.utils import secret_key
    SECRET_KEY = secret_key.generate_key()

# SECRET_KEY = '_n&qt--*3^1r--*$j6tk0^9t1^j_%134mm2frw_!!@3b3v56@1'
SESSION_SERIALIZER = 'django.contrib.sessions.serializers.PickleSerializer'

ALLOWED_HOSTS = [
                 '*',
                 ]

SITE_BRANDING = 'VSM Dashboard'

WEBROOT = '/dashboard'
LOGIN_URL = WEBROOT + '/auth/login/'
LOGOUT_URL = WEBROOT + '/auth/logout/'
# LOGIN_REDIRECT_URL can be used as an alternative for
# HORIZON_CONFIG.user_home, if user_home is not set.
# Do not set it to '/home/', as this will cause circular redirect loop
LOGIN_REDIRECT_URL = WEBROOT
SHELL_IN_A_BOX_URL = "https://10.239.44.50/dashboard/vsm_controller/"
MEDIA_ROOT = os.path.abspath(os.path.join(ROOT_PATH, '..', 'media'))
MEDIA_URL = '/media/'
STATIC_ROOT = os.path.abspath(os.path.join(ROOT_PATH, '..', 'static'))
STATIC_URL = '/static/'
ADMIN_MEDIA_PREFIX = '/static/admin/'

ROOT_URLCONF = 'vsm_dashboard.urls'

HORIZON_CONFIG = {
    'dashboards': ('vsm',),
    'default_dashboard': 'vsm',
    'user_home': None,
    'ajax_queue_limit': 10,
    'auto_fade_alerts': {
        'delay': 3000,
        'fade_duration': 1500,
        'types': ['alert-success', 'alert-info']
    },
    'help_url': "dashboard/vsm",
    'exceptions': {'recoverable': exceptions.RECOVERABLE,
                   'not_found': exceptions.NOT_FOUND,
                   'unauthorized': exceptions.UNAUTHORIZED},
}

# Set to True to allow users to upload images to glance via Horizon server.
# When enabled, a file form field will appear on the create image form.
# See documentation for deployment considerations.
HORIZON_IMAGES_ALLOW_UPLOAD = True

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'horizon.middleware.HorizonMiddleware',
    'django.middleware.doc.XViewMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

TEMPLATE_CONTEXT_PROCESSORS = (
    'django.core.context_processors.debug',
    'django.core.context_processors.i18n',
    'django.core.context_processors.request',
    'django.core.context_processors.media',
    'django.core.context_processors.static',
    'django.contrib.messages.context_processors.messages',
    'horizon.context_processors.horizon',
    'vsm_dashboard.context_processors.openstack',
)

TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
    'horizon.loaders.TemplateLoader'
)

TEMPLATE_DIRS = (
    os.path.join(ROOT_PATH, 'templates'),
)

STATICFILES_FINDERS = (
    'compressor.finders.CompressorFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
)

less_binary = os.path.join(BIN_DIR, 'lessc')
scss_binary = os.path.join('/usr', 'local', 'bin', 'scss')
COMPRESS_PRECOMPILERS = (
    ('text/less', (less_binary + ' {infile} {outfile}')),
    ('text/scss', (scss_binary + ' {infile} {outfile}')),
)

COMPRESS_CSS_FILTERS = (
    'compressor.filters.css_default.CssAbsoluteFilter',
)

COMPRESS_ENABLED = True
COMPRESS_OUTPUT_DIR = 'dashboard'
COMPRESS_CSS_HASHING_METHOD = 'hash'
COMPRESS_PARSER = 'compressor.parser.HtmlParser'
COMPRESS_OFFLINE = True

INSTALLED_APPS = (
    'vsm_dashboard',
    'django.contrib.contenttypes',
    'django.contrib.auth',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.humanize',
    'compressor',
    'horizon',
    #'vsm_dashboard.dashboards.project',
    #'vsm_dashboard.dashboards.admin',
    'vsm_dashboard.dashboards.vsm',
    'openstack_auth',
)

TEST_RUNNER = 'django_nose.NoseTestSuiteRunner'
AUTHENTICATION_BACKENDS = ('openstack_auth.backend.KeystoneBackend',)
MESSAGE_STORAGE = 'django.contrib.messages.storage.cookie.CookieStorage'

SESSION_ENGINE = 'django.contrib.sessions.backends.signed_cookies'
SESSION_COOKIE_HTTPONLY = True
SESSION_EXPIRE_AT_BROWSER_CLOSE = True
SESSION_COOKIE_SECURE = False

gettext_noop = lambda s: s
LANGUAGES = (
    ('bg', gettext_noop('Bulgarian (Bulgaria)')),
    ('cs', gettext_noop('Czech')),
    ('en', gettext_noop('English')),
    ('es', gettext_noop('Spanish')),
    ('fr', gettext_noop('French')),
    ('it', gettext_noop('Italiano')),
    ('ja', gettext_noop('Japanese')),
    ('ko', gettext_noop('Korean (Korea)')),
    ('nl', gettext_noop('Dutch (Netherlands)')),
    ('pl', gettext_noop('Polish')),
    ('pt', gettext_noop('Portuguese')),
    ('pt-br', gettext_noop('Portuguese (Brazil)')),
    ('zh-cn', gettext_noop('Simplified Chinese')),
    ('zh-tw', gettext_noop('Traditional Chinese')),
)
LANGUAGE_CODE = 'en'
USE_I18N = True
USE_L10N = True
USE_TZ = True

OPENSTACK_KEYSTONE_DEFAULT_ROLE = 'Member'

DEFAULT_EXCEPTION_REPORTER_FILTER = 'horizon.exceptions.HorizonReporterFilter'

try:
    from local.local_settings import *
except ImportError:
    logging.warning("No local_settings file found.")

# Add HORIZON_CONFIG to the context information for offline compression
COMPRESS_OFFLINE_CONTEXT = {
    'STATIC_URL': STATIC_URL,
    'HORIZON_CONFIG': HORIZON_CONFIG
}

if DEBUG:
    logging.basicConfig(level=logging.DEBUG)

# Deprecation for Essex/Folsom dashboard names; remove this code in H.
_renames = (
    #('horizon.dashboards.nova', 'vsm_dashboard.dashboards.project'),
    #('horizon.dashboards.syspanel', 'vsm_dashboard.dashboards.admin'),
    ('horizon.dashboards.vsm', 'vsm_dashboard.dashboards.vsm'),
)

INSTALLED_APPS = list(INSTALLED_APPS)
_dashboards = list(HORIZON_CONFIG['dashboards'])
_dashboards = ['vsm']

for old, new in _renames:
    if old in INSTALLED_APPS:
        warnings.warn('The "%s" package is deprecated. Please update your '
                      'INSTALLED_APPS setting to use the new "%s" package.\n'
                      % (old, new), Warning)
        INSTALLED_APPS[INSTALLED_APPS.index(old)] = new
    _old_name = old.split(".")[-1]
    if _old_name in HORIZON_CONFIG['dashboards'] and _old_name != "settings":
        _new_name = new.split(".")[-1]
        warnings.warn('The "%s" dashboard name is deprecated. Please update '
                      'your HORIZON_CONFIG["dashboards"] setting to use the '
                      'new "%s" dashboard name.\n' % (_old_name, _new_name),
                      Warning)
        _dashboards[_dashboards.index(_old_name)] = _new_name
HORIZON_CONFIG['dashboards'] = _dashboards




#!/usr/bin/env python
# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2012 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2012 Nebula, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

from distutils.core import setup
from distutils.command.install import INSTALL_SCHEMES

from vsm_dashboard.openstack.common import setup as os_common_setup

requires = os_common_setup.parse_requirements()
depend_links = os_common_setup.parse_dependency_links()
tests_require = os_common_setup.parse_requirements(['tools/test-requires'])
project = 'vsm-dashboard'

ROOT = os.path.dirname(__file__)
target_dirs = ['vsm_dashboard', 'bin', 'static']

def read(fname):
    return open(os.path.join(ROOT, fname)).read()

def split(path, result=None):
    """
    Split a path into components in a platform-neutral way.
    """
    if result is None:
        result = []
    head, tail = os.path.split(path)
    if head == '':
        return [tail] + result
    if head == path:
        return result
    return split(head, [tail] + result)

# Tell distutils not to put the data_files in platform-specific installation
# locations. See here for an explanation:
# https://groups.google.com/forum/#!topic/comp.lang.python/Nex7L-026uw
for scheme in INSTALL_SCHEMES.values():
    scheme['data'] = scheme['purelib']

# Compile the list of packages available, because distutils doesn't have
# an easy way to do this.
packages, data_files = [], []
root_dir = os.path.dirname(__file__)
if root_dir != '':
    os.chdir(root_dir)

for target_dir in target_dirs:
    for dirpath, dirnames, filenames in os.walk(target_dir):
        # Ignore dirnames that start with '.'
        for i, dirname in enumerate(dirnames):
            if dirname.startswith('.'):
                del dirnames[i]
        if '__init__.py' in filenames:
            packages.append('.'.join(split(dirpath)))
        elif filenames:
            data_files.append([dirpath, [os.path.join(dirpath, f)
                               for f in filenames]])

setup(name=project,
      version='2.0.0',
      url='https://github.com/01org/virtual-storage-manager/',
      license='Apache License 2.0',
      description="The VSM Dashboard.",
      long_description="Dakshboard for VSM Web UI",
      author='VSM Contributors',
      author_email='ml-node+s33411n1h81@n7.nabble.com',
      packages=packages,
      data_files=data_files,
      cmdclass=os_common_setup.get_cmdclass(),
      include_package_data=True,
      install_requires=requires,
      tests_require=tests_require,
      dependency_links=depend_links,
      zip_safe=False,
      classifiers=['Development Status :: 5 - Production/Stable',
                   'Framework :: Django',
                   'Intended Audience :: Developers',
                   'License :: OSI Approved :: Apache Software License',
                   'Operating System :: OS Independent',
                   'Programming Language :: Python',
                   'Topic :: Internet :: WWW/HTTP',
                   'Environment :: OpenStack']
)



# Copyright 2012 Nebula, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import glanceclient.exc as glance_exceptions
from keystoneclient import exceptions as keystone_exceptions
from novaclient import exceptions as nova_exceptions
from quantumclient.common import exceptions as quantum_exceptions
from swiftclient import client as swift_exceptions
from vsmclient import exceptions as vsm_exceptions

from .utils import TestDataContainer

def create_stubbed_exception(cls, status_code=500):
    msg = "Expected failure."

    def fake_init_exception(self, code, message, **kwargs):
        self.code = code
        self.message = message

    def fake_str(self):
        return str(self.message)

    def fake_unicode(self):
        return unicode(self.message)

    cls.__init__ = fake_init_exception
    cls.__str__ = fake_str
    cls.__unicode__ = fake_unicode
    cls.silence_logging = True
    return cls(status_code, msg)

def data(TEST):
    TEST.exceptions = TestDataContainer()

    unauth = keystone_exceptions.Unauthorized
    TEST.exceptions.keystone_unauthorized = create_stubbed_exception(unauth)

    keystone_exception = keystone_exceptions.ClientException
    TEST.exceptions.keystone = create_stubbed_exception(keystone_exception)

    nova_exception = nova_exceptions.ClientException
    TEST.exceptions.nova = create_stubbed_exception(nova_exception)

    nova_unauth = nova_exceptions.Unauthorized
    TEST.exceptions.nova_unauthorized = create_stubbed_exception(nova_unauth)

    glance_exception = glance_exceptions.ClientException
    TEST.exceptions.glance = create_stubbed_exception(glance_exception)

    quantum_exception = quantum_exceptions.QuantumClientException
    TEST.exceptions.quantum = create_stubbed_exception(quantum_exception)

    swift_exception = swift_exceptions.ClientException
    TEST.exceptions.swift = create_stubbed_exception(swift_exception)

    vsm_exception = vsm_exceptions.BadRequest
    TEST.exceptions.vsm = create_stubbed_exception(vsm_exception)




# Copyright 2014 Intel Corporation, All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import math
import logging
from vsm_dashboard.common.horizon.summary import SummaryRenderer
from vsm_dashboard.api import vsm as vsmapi
from django.utils.datastructures import SortedDict
from django.core.urlresolvers import reverse_lazy
from django.utils.safestring import mark_safe
LOG = logging.getLogger(__name__)
from vsm_dashboard.utils import get_time_delta
from vsm_dashboard.utils import get_time_delta2

def osd():
    osd_summary = vsmapi.osd_summary(None)
    osd_summary_dict = {
         "epoch":osd_summary.epoch
        ,"total":osd_summary.num_osds
        ,"up": osd_summary.num_up_osds
        ,"in": osd_summary.num_in_osds
        ,"update":get_time_delta(osd_summary.updated_at)
    }
    return osd_summary_dict

def monitor():
    monitor_summary = vsmapi.monitor_summary(None)
    monitor_summary_dict = {
         "monmap_epoch":monitor_summary.monmap_epoch
        ,"monitors":monitor_summary.monitors
        ,"election_epoch": monitor_summary.election_epoch
        ,"quorum": monitor_summary.quorum
        ,"update":get_time_delta(monitor_summary.updated_at)
    }
    return monitor_summary_dict


def mds():
    mds_summary = vsmapi.mds_summary(None)
    mds_summary_dict = {
         "epoch":mds_summary.epoch
        ,"up": mds_summary.num_up_mdses
        ,"in": mds_summary.num_in_mdses
        ,"max": mds_summary.num_max_mdses
        ,"failed":mds_summary.num_failed_mdses
        ,"stopped":mds_summary.num_stopped_mdses
    }
    return mds_summary_dict

def objects():
    pg_summary = vsmapi.placement_group_summary(None)

    pg_summary_dict = {
         "degraded_objects":pg_summary.degraded_objects
        ,"degraded_total": pg_summary.degraded_total
        ,"degraded_ratio": pg_summary.degraded_ratio
        ,"unfound_objects": pg_summary.unfound_objects
        ,"unfound_total": pg_summary.unfound_total
        ,"unfound_ratio": pg_summary.unfound_ratio
    }
    return pg_summary_dict

def performance():
    pg_summary = vsmapi.placement_group_summary(None)
  
    pg_summary_dict = {
         "read": "%s B/s" % pg_summary.read_bytes_sec
        ,"write": "%s B/s" % pg_summary.write_bytes_sec
        ,"operations":"%s operation/s" % pg_summary.op_per_sec
    }
    return pg_summary_dict 

def pg():
    pg_summary = vsmapi.placement_group_summary(None)

    #get the pag collection
    # pg_state_sort = SortedDict()
    # for pgs in pg_summary.pgs_by_state:
    #     pg_state_sort["PGs " + pgs['state_name']] = pgs['count']

    pg_summary_dict = {
         "pgmap_version":pg_summary.version
        ,"total_pgs":pg_summary.num_pgs
        #,"pg_state":pg_state_sort
        ,"update":get_time_delta(pg_summary.updated_at)
    }
    return pg_summary_dict

def format_data_size(data_size):
    """ Return a formatted string in bytes, MB, GB, or TB, not allowing any value to go over 1000.
    :param data_bytes: the integer number of bytes to be formatted.
    :return: the string formatted with a proper units suffix.
    """
    if not data_size:
        return str(0)
    if data_size > 1:
        units = [" B", " KiB", " MiB", " GiB", " TiB", " PiB", " EiB", " ZiB", " YiB"]
        idx = int(math.ceil(math.log(data_size, 1000))) - 1
        val = data_size / (math.pow(1024, idx))
        return str(round(val, 2)) + units[idx]
    return "1 B"

def capactiy():
    pg_summary = vsmapi.placement_group_summary(None)
    capactiy_summary_dict = {
         "data_used": format_data_size(pg_summary.data_bytes)
        ,"total_used": format_data_size(pg_summary.bytes_used)
        ,"available": format_data_size(pg_summary.bytes_avail)
        ,"total":  format_data_size(pg_summary.bytes_total)
    }
    return capactiy_summary_dict


class StorageGroupSummary(SummaryRenderer):
    verbose_name = "Storage Group Summary"
    name = "storage_group_summary"
    detail = {'name': "Storage Group Status",
              'link': reverse_lazy("horizon:vsm:storage-group-status:index")}

    def get_summary(self):

        _sgs = vsmapi.storage_group_status(self.request,)
        _cfg = {"storage_group_near_full_threshold":65,
            "storage_group_full_threshold":85,}
        _num = 0
        _num_near_full = 0
        _num_full = 0
        for _sg in _sgs:
            _sg.capacity_total = 1 if not _sg.capacity_total else _sg.capacity_total
            capcity_percent_used = 0 if not _sg.capacity_total else _sg.capacity_used * 100 / _sg.capacity_total
            if capcity_percent_used <_cfg["storage_group_near_full_threshold"]:
                pass
            elif capcity_percent_used < _cfg["storage_group_full_threshold"]:
                _num_near_full += 1
            else:
                _num_full += 1
            _num += 1

        data = SortedDict()
        data["Total Storage Groups"] = _num
        data["Storage Groups Near Full"] = _num_near_full
        data["Storage Groups Full"] = _num_full
        try:
            data["Last Updated"] = get_time_delta(_sgs[0].updated_at)
        except:
            pass
        return data

class VsmSummary(SummaryRenderer):
    # TODO
    #verbose_name = "VSM Status" + " Version: 2015.03-1.2"
    verbose_name = "VSM Status"
    name = "vsm_summary"

    def get_summary(self):
        vsm_summary = vsmapi.vsm_summary(self.request)
        data = SortedDict()
        data["Version"] = "2015.03-1.2"
        data["Uptime"] = get_time_delta2(vsm_summary.created_at)
        return data
        try:
            if not vsm_summary.is_ceph_active:
                data["Warning"] = mark_safe("<span style='color:red'>Cluster not responding <br> to Ceph Status</span>")
        except Exception, e:
            pass
        #data["Average Response Time"] = 123
        return data

class PGSummary(SummaryRenderer):
    # TODO
    verbose_name = "PG Summary"
    name = "pg_summary"
    detail = {'name': "PG Status",
              'link': reverse_lazy("horizon:vsm:pg-status:index")}

    def get_summary(self):
        pg_summary = vsmapi.placement_group_summary(self.request)
        LOG.info('pg_summary:%s'%pg_summary)
        #LOG.error(pg_summary)
        #LOG.error('pg_summary>')
        data = SortedDict()
        data["PGmap Version"] = pg_summary.version
        data["Total PGs"] = pg_summary.num_pgs
        for pgs in pg_summary.pgs_by_state:
            data["PGs " + pgs['state_name']] = pgs['count']
        data["Last Updated"] = get_time_delta(pg_summary.updated_at)
        return data

class PGSummaryDashboard(SummaryRenderer):
    verbose_name = "PG Summary"
    name = "pg_summary"
    detail = {'name': "PG Status",
              'link': reverse_lazy("horizon:vsm:pg-status:index")}

    def get_summary(self):
        pg_summary = vsmapi.placement_group_summary(self.request)
        #LOG.error('<pg_summary')
        #LOG.error(pg_summary)
        #LOG.error(dir(pg_summary))
        #LOG.error('pg_summary>')
        data = SortedDict()
        data["PGmap Version"] = pg_summary.version
        data["Total PGs"] = pg_summary.num_pgs
        data["PGs active+clean"] = sum([pgs['count'] for pgs in pg_summary.pgs_by_state
                                        if pgs['state_name'] == "active+clean"])
        data["PGs not active+clean"] = sum([pgs['count'] for pgs in pg_summary.pgs_by_state
                                        if pgs['state_name'] != "active+clean"])
        data["Last Updated"] = get_time_delta(pg_summary.updated_at)
        return data

class ClusterSummary(SummaryRenderer):
    # TODO
    verbose_name = "Cluster Summary"
    name = "cluster_summary"
    font = '<span id="%s"> %s </span>'
    css_id = 'health_ok'

    def get_summary(self):
        cluster_summary = vsmapi.cluster_summary(self.request)
        cluster_name = vsmapi.get_cluster_list(self.request)[1]['clusters'][0]['name']
        data = SortedDict()
        data["Cluster Name"] = cluster_name
        try:
            data["Status"] = cluster_summary.health_list[0]
            status = data['Status'].lower()
            if status.find('ok') != -1:
                self.css_id = "health_ok"
            elif status.find('war') != -1:
                self.css_id = "health_warning"
            elif status.find('err') != -1:
                self.css_id = "health_error"

            msg = self.font % (self.css_id, data["Status"])
            data["Status"] = mark_safe(msg)
        except:
            pass
        return data

class WarningSummary(SummaryRenderer):
    # TODO
    verbose_name = "Warning and Errors"
    name = "waring_summary"

    def get_summary(self):
        cluster_summary = vsmapi.cluster_summary(self.request)
        data = SortedDict()
        for k, d in enumerate(cluster_summary.detail):
            data[mark_safe("<lable id=detail-%s>Warn</lable>" % k)] = d
        for k, d in enumerate(cluster_summary.status):
            try:
                data[mark_safe("<lable id=summary-%s>%s</lable>" % (k, d.get('severity')))] = d.get('summary')
            except:
                pass
        return data

class CapacitySummary(SummaryRenderer):
    verbose_name = "Capacity Summary"
    name = "capacity_summary"

    def get_summary(self):
        pg_summary = vsmapi.placement_group_summary(self.request)
        data = SortedDict()
        data['Data Capacity Used'] = format_data_size(pg_summary.data_bytes)
        data['Total Capacity Used'] = format_data_size(pg_summary.bytes_used)
        data['Capacity available'] = format_data_size(pg_summary.bytes_avail)
        data['Capacity Total'] = format_data_size(pg_summary.bytes_total)
        return data

class PerformanceSummary(SummaryRenderer):
    verbose_name = "Performance Summary"
    name = "performance_summary"

    def get_summary(self):
        pg_summary = vsmapi.placement_group_summary(self.request)
        data = SortedDict()
        data['Total Reads'] = "%s B/s" % pg_summary.read_bytes_sec
        data['Total Writes'] = "%s B/s" % pg_summary.write_bytes_sec
        data['Total Operations'] = "%s operation/s" % pg_summary.op_per_sec
        return data

class ClusterHealthSummary(SummaryRenderer):
    # TODO
    verbose_name = "Cluster Health Summary"
    name = "cluster_health_summary"

    def get_summary_list(self):
        cluster_summary = vsmapi.cluster_summary(self.request)
        summary_list = []
        for d in cluster_summary.health_list[1:]:
            summary_list.append(d)
        return summary_list




# vim: tabstop=4 shiftwidth=4 softtabstop=4

#    Copyright 2013, Big Switch Networks, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from mox import IsA
from django import http

from vsm_dashboard import api
from vsm_dashboard.test import helpers as test
from vsm_dashboard.api.lbaas import Vip, Pool, Member, PoolMonitor

from quantumclient.v2_0.client import Client as quantumclient

class LbaasApiTests(test.APITestCase):
    @test.create_stubs({quantumclient: ('create_vip',)})
    def test_vip_create(self):
        vip1 = self.api_vips.first()
        form_data = {'address': vip1['address'],
                     'name': vip1['name'],
                     'description': vip1['description'],
                     'subnet_id': vip1['subnet_id'],
                     'protocol_port': vip1['protocol_port'],
                     'protocol': vip1['protocol'],
                     'pool_id': vip1['pool_id'],
                     'session_persistence': vip1['session_persistence'],
                     'connection_limit': vip1['connection_limit'],
                     'admin_state_up': vip1['admin_state_up']
                     }
        vip = {'vip': self.api_vips.first()}
        quantumclient.create_vip({'vip': form_data}).AndReturn(vip)
        self.mox.ReplayAll()

        ret_val = api.lbaas.vip_create(self.request, **form_data)
        self.assertIsInstance(ret_val, api.lbaas.Vip)

    @test.create_stubs({quantumclient: ('list_vips',)})
    def test_vips_get(self):
        vips = {'vips': [{'id': 'abcdef-c3eb-4fee-9763-12de3338041e',
                          'address': '10.0.0.100',
                          'name': 'vip1name',
                          'description': 'vip1description',
                          'subnet_id': '12381d38-c3eb-4fee-9763-12de3338041e',
                          'protocol_port': '80',
                          'protocol': 'HTTP',
                          'pool_id': '8913dde8-4915-4b90-8d3e-b95eeedb0d49',
                          'connection_limit': '10',
                          'admin_state_up': True
                          }, ]}
        quantumclient.list_vips().AndReturn(vips)
        self.mox.ReplayAll()

        ret_val = api.lbaas.vips_get(self.request)
        for v in ret_val:
            self.assertIsInstance(v, api.lbaas.Vip)

    @test.create_stubs({quantumclient: ('show_vip',)})
    def test_vip_get(self):
        vip = {'vip': {'id': 'abcdef-c3eb-4fee-9763-12de3338041e',
                       'address': '10.0.0.100',
                       'name': 'vip1name',
                       'description': 'vip1description',
                       'subnet_id': '12381d38-c3eb-4fee-9763-12de3338041e',
                       'protocol_port': '80',
                       'protocol': 'HTTP',
                       'pool_id': '8913dde8-4915-4b90-8d3e-b95eeedb0d49',
                       'connection_limit': '10',
                       'admin_state_up': True
                       }}
        quantumclient.show_vip(vip['vip']['id']).AndReturn(vip)
        self.mox.ReplayAll()

        ret_val = api.lbaas.vip_get(self.request, vip['vip']['id'])
        self.assertIsInstance(ret_val, api.lbaas.Vip)

    @test.create_stubs({quantumclient: ('update_vip',)})
    def test_vip_update(self):
        form_data = {'address': '10.0.0.100',
                     'name': 'vip1name',
                     'description': 'vip1description',
                     'subnet_id': '12381d38-c3eb-4fee-9763-12de3338041e',
                     'protocol_port': '80',
                     'protocol': 'HTTP',
                     'pool_id': '8913dde8-4915-4b90-8d3e-b95eeedb0d49',
                     'connection_limit': '10',
                     'admin_state_up': True
                     }

        vip = {'vip': {'id': 'abcdef-c3eb-4fee-9763-12de3338041e',
                       'address': '10.0.0.100',
                       'name': 'vip1name',
                       'description': 'vip1description',
                       'subnet_id': '12381d38-c3eb-4fee-9763-12de3338041e',
                       'protocol_port': '80',
                       'protocol': 'HTTP',
                       'pool_id': '8913dde8-4915-4b90-8d3e-b95eeedb0d49',
                       'connection_limit': '10',
                       'admin_state_up': True
                       }}
        quantumclient.update_vip(vip['vip']['id'], form_data).AndReturn(vip)
        self.mox.ReplayAll()

        ret_val = api.lbaas.vip_update(self.request,
                                       vip['vip']['id'], **form_data)
        self.assertIsInstance(ret_val, api.lbaas.Vip)

    @test.create_stubs({quantumclient: ('create_pool',)})
    def test_pool_create(self):
        form_data = {'name': 'pool1name',
                     'description': 'pool1description',
                     'subnet_id': '12381d38-c3eb-4fee-9763-12de3338041e',
                     'protocol': 'HTTP',
                     'lb_method': 'ROUND_ROBIN',
                     'admin_state_up': True
                     }

        pool = {'pool': {'id': 'abcdef-c3eb-4fee-9763-12de3338041e',
                         'name': 'pool1name',
                         'description': 'pool1description',
                         'subnet_id': '12381d38-c3eb-4fee-9763-12de3338041e',
                         'protocol': 'HTTP',
                         'lb_method': 'ROUND_ROBIN',
                         'admin_state_up': True
                         }}
        quantumclient.create_pool({'pool': form_data}).AndReturn(pool)
        self.mox.ReplayAll()

        ret_val = api.lbaas.pool_create(self.request, **form_data)
        self.assertIsInstance(ret_val, api.lbaas.Pool)

    @test.create_stubs({quantumclient: ('list_pools',)})
    def test_pools_get(self):
        pools = {'pools': [{
                    'id': 'abcdef-c3eb-4fee-9763-12de3338041e',
                    'name': 'pool1name',
                    'description': 'pool1description',
                    'subnet_id': '12381d38-c3eb-4fee-9763-12de3338041e',
                    'protocol': 'HTTP',
                    'lb_method': 'ROUND_ROBIN',
                    'admin_state_up': True}, ]}
        quantumclient.list_pools().AndReturn(pools)
        self.mox.ReplayAll()

        ret_val = api.lbaas.pools_get(self.request)
        for v in ret_val:
            self.assertIsInstance(v, api.lbaas.Pool)

    @test.create_stubs({quantumclient: ('show_pool',)})
    def test_pool_get(self):
        pool = {'pool': {'id': 'abcdef-c3eb-4fee-9763-12de3338041e',
                         'name': 'pool1name',
                         'description': 'pool1description',
                         'subnet_id': '12381d38-c3eb-4fee-9763-12de3338041e',
                         'protocol': 'HTTP',
                         'lb_method': 'ROUND_ROBIN',
                         'admin_state_up': True
                         }}
        quantumclient.show_pool(pool['pool']['id']).AndReturn(pool)
        self.mox.ReplayAll()

        ret_val = api.lbaas.pool_get(self.request, pool['pool']['id'])
        self.assertIsInstance(ret_val, api.lbaas.Pool)

    @test.create_stubs({quantumclient: ('update_pool',)})
    def test_pool_update(self):
        form_data = {'name': 'pool1name',
                     'description': 'pool1description',
                     'subnet_id': '12381d38-c3eb-4fee-9763-12de3338041e',
                     'protocol': 'HTTPS',
                     'lb_method': 'LEAST_CONNECTION',
                     'admin_state_up': True
                     }

        pool = {'pool': {'id': 'abcdef-c3eb-4fee-9763-12de3338041e',
                         'name': 'pool1name',
                         'description': 'pool1description',
                         'subnet_id': '12381d38-c3eb-4fee-9763-12de3338041e',
                         'protocol': 'HTTPS',
                         'lb_method': 'LEAST_CONNECTION',
                         'admin_state_up': True
                         }}
        quantumclient.update_pool(pool['pool']['id'],
                                  form_data).AndReturn(pool)
        self.mox.ReplayAll()

        ret_val = api.lbaas.pool_update(self.request,
                                        pool['pool']['id'], **form_data)
        self.assertIsInstance(ret_val, api.lbaas.Pool)

    @test.create_stubs({quantumclient: ('create_health_monitor',
                                        'associate_health_monitor')})
    def test_pool_health_monitor_create(self):
        form_data = {'type': 'PING',
                     'delay': '10',
                     'timeout': '10',
                     'max_retries': '10',
                     'http_method': 'GET',
                     'url_path': '/monitor',
                     'expected_codes': '200',
                     'admin_state_up': True
                     }
        form_data_with_pool_id = {
            'pool_id': 'abcdef-c3eb-4fee-9763-12de3338041e',
            'type': 'PING',
            'delay': '10',
            'timeout': '10',
            'max_retries': '10',
            'http_method': 'GET',
            'url_path': '/monitor',
            'expected_codes': '200',
            'admin_state_up': True}
        monitor = {'health_monitor': {
                'id': 'abcdef-c3eb-4fee-9763-12de3338041e',
                'type': 'PING',
                'delay': '10',
                'timeout': '10',
                'max_retries': '10',
                'http_method': 'GET',
                'url_path': '/monitor',
                'expected_codes': '200',
                'admin_state_up': True}}
        monitor_id = {'health_monitor': {
                'id': 'abcdef-c3eb-4fee-9763-12de3338041e'}}
        quantumclient.create_health_monitor({
                'health_monitor': form_data}).AndReturn(monitor)
        quantumclient.associate_health_monitor(
            form_data_with_pool_id['pool_id'], monitor_id)
        self.mox.ReplayAll()

        ret_val = api.lbaas.pool_health_monitor_create(
            self.request, **form_data_with_pool_id)
        self.assertIsInstance(ret_val, api.lbaas.PoolMonitor)

    @test.create_stubs({quantumclient: ('list_health_monitors',)})
    def test_pool_health_monitors_get(self):
        monitors = {'health_monitors': [
                {'id': 'abcdef-c3eb-4fee-9763-12de3338041e',
                 'type': 'PING',
                 'delay': '10',
                 'timeout': '10',
                 'max_retries': '10',
                 'http_method': 'GET',
                 'url_path': '/monitor',
                 'expected_codes': '200',
                 'admin_state_up': True}, ]}

        quantumclient.list_health_monitors().AndReturn(monitors)
        self.mox.ReplayAll()

        ret_val = api.lbaas.pool_health_monitors_get(self.request)
        for v in ret_val:
            self.assertIsInstance(v, api.lbaas.PoolMonitor)

    @test.create_stubs({quantumclient: ('show_health_monitor',)})
    def test_pool_health_monitor_get(self):
        monitor = {'health_monitor':
                   {'id': 'abcdef-c3eb-4fee-9763-12de3338041e',
                    'type': 'PING',
                    'delay': '10',
                    'timeout': '10',
                    'max_retries': '10',
                    'http_method': 'GET',
                    'url_path': '/monitor',
                    'expected_codes': '200',
                    'admin_state_up': True}}
        quantumclient.show_health_monitor(
            monitor['health_monitor']['id']).AndReturn(monitor)
        self.mox.ReplayAll()

        ret_val = api.lbaas.pool_health_monitor_get(
            self.request, monitor['health_monitor']['id'])
        self.assertIsInstance(ret_val, api.lbaas.PoolMonitor)

    @test.create_stubs({quantumclient: ('create_member', )})
    def test_member_create(self):
        form_data = {'pool_id': 'abcdef-c3eb-4fee-9763-12de3338041e',
                     'address': '10.0.1.2',
                     'protocol_port': '80',
                     'weight': '10',
                     'admin_state_up': True
                     }

        member = {'member':
                  {'id': 'abcdef-c3eb-4fee-9763-12de3338041e',
                   'pool_id': 'abcdef-c3eb-4fee-9763-12de3338041e',
                   'address': '10.0.1.2',
                   'protocol_port': '80',
                   'weight': '10',
                   'admin_state_up': True}}

        quantumclient.create_member({'member': form_data}).AndReturn(member)
        self.mox.ReplayAll()

        ret_val = api.lbaas.member_create(self.request, **form_data)
        self.assertIsInstance(ret_val, api.lbaas.Member)

    @test.create_stubs({quantumclient: ('list_members',)})
    def test_members_get(self):
        members = {'members': [
                {'id': 'abcdef-c3eb-4fee-9763-12de3338041e',
                 'pool_id': 'abcdef-c3eb-4fee-9763-12de3338041e',
                 'address': '10.0.1.2',
                 'protocol_port': '80',
                 'weight': '10',
                 'admin_state_up': True
                 }, ]}
        quantumclient.list_members().AndReturn(members)
        self.mox.ReplayAll()

        ret_val = api.lbaas.members_get(self.request)
        for v in ret_val:
            self.assertIsInstance(v, api.lbaas.Member)

    @test.create_stubs({quantumclient: ('show_member',)})
    def test_member_get(self):
        member = {'member': {'id': 'abcdef-c3eb-4fee-9763-12de3338041e',
                              'pool_id': 'abcdef-c3eb-4fee-9763-12de3338041e',
                              'address': '10.0.1.2',
                              'protocol_port': '80',
                              'weight': '10',
                              'admin_state_up': True}}
        quantumclient.show_member(member['member']['id']).AndReturn(member)
        self.mox.ReplayAll()

        ret_val = api.lbaas.member_get(self.request, member['member']['id'])
        self.assertIsInstance(ret_val, api.lbaas.Member)

    @test.create_stubs({quantumclient: ('update_member',)})
    def test_member_update(self):
        form_data = {'pool_id': 'abcdef-c3eb-4fee-9763-12de3338041e',
                     'address': '10.0.1.4',
                     'protocol_port': '80',
                     'weight': '10',
                     'admin_state_up': True
                     }

        member = {'member': {'id': 'abcdef-c3eb-4fee-9763-12de3338041e',
                             'pool_id': 'abcdef-c3eb-4fee-9763-12de3338041e',
                             'address': '10.0.1.2',
                             'protocol_port': '80',
                             'weight': '10',
                             'admin_state_up': True
                             }}

        quantumclient.update_member(member['member']['id'],
                                    form_data).AndReturn(member)
        self.mox.ReplayAll()

        ret_val = api.lbaas.member_update(self.request,
                                          member['member']['id'], **form_data)
        self.assertIsInstance(ret_val, api.lbaas.Member)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2012 Nebula, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from horizon.test import helpers as test

class BrowserTests(test.SeleniumTestCase):
    def test_splash(self):
        self.selenium.get(self.live_server_url)
        button = self.selenium.find_element_by_tag_name("button")
        self.assertEqual(button.text, "Sign In")



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Corporation, All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import division
import logging,summarys,json,commands,time
from django.http import HttpResponse,HttpResponseRedirect
from django.shortcuts import render
from vsm_dashboard.api import vsm as vsmapi
from vsm_dashboard.utils import get_time_delta

LOG = logging.getLogger(__name__)

def index(request):
    pool_status = vsmapi.pool_status(None)
    server_list = vsmapi.get_server_list(None)
    status = [server.status for server in server_list]
    if len(pool_status) != 0 or 'Active' in status:
        return render(request,'vsm/overview/index.html',{})
    else:
        return HttpResponseRedirect("/dashboard/vsm/clustermgmt/")

#show osd summary
def osd_summary(request):
    return HttpResponse(json.dumps(summarys.osd()))

#show monitor summary
def monitor_summary(request):
    return HttpResponse(json.dumps(summarys.monitor()))

#show mds summary
def mds_summary(request):
    return HttpResponse(json.dumps(summarys.mds()))

#show objects summary
def objects_summary(request):
    return HttpResponse(json.dumps(summarys.objects()))

#show performance summary
def performance_summary(request):
    return HttpResponse(json.dumps(summarys.performance()))

#show pg summary
def pg_summary(request):
    return HttpResponse(json.dumps(summarys.pg()))

#show capacity summary
def capacity_summary(request):
    return HttpResponse(json.dumps(summarys.capactiy()))



#handle the vsm_version
def version(request):
    return HttpResponse(get_version())

#handle the cluster data
def cluster(request):
    return HttpResponse(get_cluster())

#handle the capactiy data
def capacity(request):
    return HttpResponse(get_capacity())

#handle the OSD data
def OSD(request):
    return HttpResponse(get_OSD())

#handle the monitor data
def monitor(request):
    return HttpResponse(get_monitor())

#handle the MDS data
def MDS(request):
    return HttpResponse(get_MDS())

#handle the storage data
def storage(request):
    return HttpResponse(get_storage())

#handle the pg data
def PG(request):
    return HttpResponse(get_PG())

def IOPS(request):
    if request.body:
        return HttpResponse(get_performance_IOPs(request))

def latency(request):
    # data = json.loads(request.body)
    # timestamp = int(data["timestamp"])
    if request.body:
        return HttpResponse(get_performance_Latency(request))

def bandwidth(request):
    # data = json.loads(request.body)
    # timestamp = int(data["timestamp"])
    if request.body:
        return HttpResponse(get_performance_Bandwith(request))


def CPU(request):
    if request.body:
        return HttpResponse(get_performance_cpu(request))

def perf_enabled(request):
    return HttpResponse(get_performance_enabled(request))

def get_vsm_version():
    try:
        (status, out) = commands.getstatusoutput('vsm --version')
    except:
        out = '2.0'
    return out

#get the vsm_version
def get_version():
    ceph_version = ''
    up_time = ''
    try:
        vsm_summary = vsmapi.vsm_summary(None)
        if vsm_summary is not None:
            up_time = get_time_delta(vsm_summary.created_at)
            ceph_version = vsm_summary.ceph_version
    except:
        pass
    vsm_version = get_vsm_version()
    vsm_version = {"version": vsm_version,
                   "update": up_time,
                   "ceph_version":ceph_version,
    }
    version_data = json.dumps(vsm_version)
    return version_data

#get the cluster data
def get_cluster():
    cluster_summary = vsmapi.cluster_summary(None)
    cluster_name = vsmapi.get_cluster_list(None)[1]['clusters'][0]['name']
    #HEALTH_OK HEALTH_WARN  HEALTH_ERR
    cluster_status = cluster_summary.health_list[0]
    cluster_note = []
    for note in cluster_summary.health_list[1:]:
        cluster_note.append(note)

    vsm_status_dict = { "name":cluster_name
                      , "status": cluster_status
                      , "note":cluster_note}
    vsmstatusdata = json.dumps(vsm_status_dict)
    return vsmstatusdata

#get the capactiy data
def get_capacity():
    pg_summary = vsmapi.placement_group_summary(None)
    capactiy_used = pg_summary.bytes_used
    capactiy_total = pg_summary.bytes_total
    used_percent = ('%.2f'%((pg_summary.bytes_used/pg_summary.bytes_total)*100));
    capacity_dict = {"used":capactiy_used,"total":capactiy_total,"percent":used_percent}
    capacitydata = json.dumps(capacity_dict)
    return capacitydata

#get the OSD data
def get_OSD():
    #get the full or near full threshold
    settings = vsmapi.get_setting_dict(None)
    disk_near_full_threshold = int(settings['disk_near_full_threshold'])
    disk_full_threshold = int(settings['disk_full_threshold'])

    in_up = 0
    in_down = 0
    out_up = 0
    out_down = 0
    available_count = 0
    near_full_count = 0
    full_count = 0
    osd_summary = vsmapi.osd_summary(None)
    _osd_status = vsmapi.osd_status(None)

    for _osd in _osd_status:
        _osd_capacity_avaliable = 0 if not _osd.device['avail_capacity_kb']\
            else int(_osd.device['avail_capacity_kb']/1024)
        _osd_capacity_used = 0 if not _osd.device['used_capacity_kb']\
            else int(_osd.device['used_capacity_kb']/1024)
        _osd_capacity_total = 0 if not _osd.device['total_capacity_kb']\
            else int(_osd.device['total_capacity_kb']/1024)

        if _osd_capacity_total and _osd.state in ["In-Up", "In-Down", "Out-Up", "Out-Down", "Out-Down-Autoout"]:
            _osd_capacity_status = round(_osd_capacity_used * 1.0 / _osd_capacity_total * 100, 2)
            if _osd_capacity_status >= disk_full_threshold:
                full_count = full_count + 1
            elif _osd_capacity_status >= disk_near_full_threshold:
                near_full_count = near_full_count + 1
            else:
                available_count = available_count + 1

        if _osd.state == "In-Up":
            in_up=in_up+1
        elif _osd.state == "In-Down":
            in_down=in_down+1
        elif _osd.state == "Out-Up":
            out_up=out_up+1
        elif _osd.state == "Out-Down" or _osd.state == "Out-Down-Autoout":
            out_down=out_down+1

    OSD_dict = {"epoch":osd_summary.epoch
              ,"update":get_time_delta(osd_summary.updated_at)
              ,"in_up":in_up
              ,"in_down":in_down
              ,"out_up":out_up
              ,"out_down":out_down
              ,"capacity_full_count":full_count
              ,"capacity_near_full_count":near_full_count
              ,"capacity_available_count":available_count
              }
    OSDdata = json.dumps(OSD_dict)
    return OSDdata

#get the monitor data
def get_monitor():
    monitor_summary = vsmapi.monitor_summary(None)
    epoch = monitor_summary.monmap_epoch
    update = get_time_delta(monitor_summary.updated_at)
    quorumlist = monitor_summary.quorum.split(" ")
    quorum_leader_rank = monitor_summary.quorum_leader_rank
    leader_list_index = quorumlist.index(quorum_leader_rank)
    #monitors = monitor_summary.monitors
    Monitor_dict = {"epoch":epoch
              ,"update":update
              ,"quorum":quorumlist
              ,"selMonitor":leader_list_index}
    Monitordata = json.dumps(Monitor_dict)
    return Monitordata

#get the MDS data
def get_MDS():
    mds_summary = vsmapi.mds_summary(None)
    ecpoch = mds_summary.epoch
    Up = mds_summary.num_up_mdses
    In = mds_summary.num_in_mdses
    Failed = mds_summary.num_failed_mdses
    Stopped = mds_summary.num_stopped_mdses
    PoolData = mds_summary.data_pools
    MetaData = mds_summary.metadata_pool

    mds_status = vsmapi.mds_status(None)
    update = ""
    for mds in mds_status:
        update = get_time_delta(mds.updated_at)

    MDS_dict = {"epoch":ecpoch
              ,"update":update
              ,"Up":Up
              ,"In":In
              ,"Failed":Failed
              ,"Stopped":Stopped
              ,"PoolData":PoolData
              ,"MetaData":MetaData}
    MDSdata = json.dumps(MDS_dict)
    return MDSdata

#get the storage data
def get_storage():
    #get the threshold for storage group
    settings = vsmapi.get_setting_dict(None)
    _sgs = vsmapi.storage_group_status(None)

    _num = 0
    _num_normal = 0
    _num_near_full = 0
    _num_full = 0
    for _sg in _sgs:
        _sg.capacity_total = 1 if not _sg.capacity_total else _sg.capacity_total
        capcity_percent_used = 0 if not _sg.capacity_total else _sg.capacity_used * 100 / _sg.capacity_total
        if capcity_percent_used > float(settings["storage_group_full_threshold"]):
             _num_full+=1
        elif capcity_percent_used > float(settings["storage_group_near_full_threshold"]):
            _num_near_full+=1
        else:
            _num_normal+=1
        _num+=1

    Storage_dict = {"nearfull":_num_near_full
                   ,"full":_num_full
                   ,"normal":_num_normal
                   ,"update":get_time_delta(_sgs[0].updated_at)}

    storagedata = json.dumps(Storage_dict)
    return storagedata

#get the PG data
def get_PG():
    pg_summary = vsmapi.placement_group_summary(None)
    version = pg_summary.version
    update = get_time_delta(pg_summary.updated_at)
    pg_total = pg_summary.num_pgs
    pg_active_clean = sum([pgs['count'] for pgs in pg_summary.pgs_by_state
                                        if pgs['state_name'] == "active+clean"])
    pg_not_active_clean = sum([pgs['count'] for pgs in pg_summary.pgs_by_state
                                        if pgs['state_name'] != "active+clean"])
    pg_dict = {"version":version
              ,"update":update
              ,"total":pg_total
              ,"active_clean":pg_active_clean
              ,"not_active_clean":pg_not_active_clean}
    pgdata = json.dumps(pg_dict)
    return pgdata

def get_performance_IOPs(request):
    data = json.loads(request.body)
    start_time = data["timestamp"] and int(data["timestamp"]) or int(time.time())-30
    end_time = None

    ops_opts = {
         "metrics_name": "iops",
         "timestamp_start": start_time,
         "timestamp_end": end_time,
         "correct_cnt": None,
    }

    #print 'test1111'
    iops_data = vsmapi.get_metrics_all_types(request,ops_opts)
    #print 'test222'
    ops_r_data = iops_data.get('osd_op_r',[])
    ops_w_data = iops_data.get('osd_op_w',[])
    ops_rw_data = iops_data.get('osd_op_rw',[])


    items = {}
    for r_metric in ops_r_data:
        items[r_metric["timestamp"]] = {"timestamp": r_metric["timestamp"], "r_value": r_metric["metrics_value"] or 0, "w_value": 0, "rw_value": 0}
    for w_metric in ops_w_data:
        if items.has_key(w_metric["timestamp"]):
            items[w_metric["timestamp"]]["w_value"] = w_metric["metrics_value"] or 0
    for rw_metric in ops_rw_data:
        if items.has_key(rw_metric["timestamp"]):
            items[rw_metric["timestamp"]]["rw_value"] = rw_metric["metrics_value"] or 0

    keys = items.keys()
    keys.sort()
    metric_list = [items[key] for key in keys]

    ops_data_dict = {"metrics": metric_list}
    ops_data = json.dumps(ops_data_dict)
    return ops_data

def get_performance_Latency(request):
    data = json.loads(request.body)
    start_time = data["timestamp"] and int(data["timestamp"]) or int(time.time())-30
    end_time = None
    latency_opts = {
         "metrics_name": "lantency",
         "timestamp_start": start_time,
         "timestamp_end": end_time,
         "correct_cnt": None,
    }
    lantency_data = vsmapi.get_metrics_all_types(request,latency_opts)
    latency_r_data = lantency_data.get('osd_op_r_latency',[])
    latency_w_data = lantency_data.get('osd_op_w_latency',[])
    latency_rw_data = lantency_data.get('osd_op_rw_latency',[])


    items = {}
    for r_metric in latency_r_data:
        items[r_metric["timestamp"]] = {"timestamp": r_metric["timestamp"], "r_value":  r_metric["metrics_value"] and round( r_metric["metrics_value"],2) or 0, "w_value": 0, "rw_value": 0}
    for w_metric in latency_w_data:
        if items.has_key(w_metric["timestamp"]):
            items[w_metric["timestamp"]]["w_value"] = w_metric["metrics_value"] and  round(w_metric["metrics_value"],2) or 0
    for rw_metric in latency_rw_data:
        if items.has_key(rw_metric["timestamp"]):
            items[rw_metric["timestamp"]]["rw_value"] = rw_metric["metrics_value"] and round(rw_metric["metrics_value"],2) or 0

    keys = items.keys()
    keys.sort()
    metric_list = [items[key] for key in keys]
    ops_data_dict = {"metrics": metric_list}
    ops_data = json.dumps(ops_data_dict)
    return ops_data


def get_performance_Bandwith(request):
    data = json.loads(request.body)
    start_time = data["timestamp"] and int(data["timestamp"]) or int(time.time())-30
    end_time = None
    bandwidth_opts = {
         "metrics_name":"bandwidth"
        ,"timestamp_start":start_time
        ,"timestamp_end":end_time
        ,"correct_cnt":None
    }

    bandwidth_data = vsmapi.get_metrics_all_types(request,bandwidth_opts)
    bandwidth_in_data = bandwidth_data.get('osd_op_in_bytes',[])
    bandwidth_out_data = bandwidth_data.get('osd_op_out_bytes',[])
    items = {}
    for in_metric in bandwidth_in_data:
        items[in_metric["timestamp"]] = {"timestamp": in_metric["timestamp"], "in_value": in_metric["metrics_value"] and round(in_metric["metrics_value"],2) or 0, "out_value": 0}
    for out_metric in bandwidth_out_data:
        if items.has_key(out_metric["timestamp"]):
            items[out_metric["timestamp"]]["out_value"] = out_metric["metrics_value"] and round(out_metric["metrics_value"],2) or 0

    keys = items.keys()
    keys.sort()
    metric_list = [items[key] for key in keys]
    ops_data_dict = {"metrics": metric_list}
    ops_data = json.dumps(ops_data_dict)
    return ops_data


def get_performance_cpu(request):
    #data = json.loads(request.body)
    start_time = int(time.time())-600
    end_time = None
    cpu_opts = {
         "metrics_name":"cpu_usage"
        ,"timestamp_start":start_time
        ,"timestamp_end":end_time
    }

    cpu_data = vsmapi.get_metrics(request,cpu_opts)["metrics"]

    cpu_data_dict = {"time":[],"cpus":[]}
    cpu_data_dict['time'] = list(set([i['timestamp'] for i in cpu_data]))
    cpu_data_dict['time'].sort()
    cpu_data_dict['time'] = cpu_data_dict['time'][-7:]
    _cpu = {}
    for _metric in cpu_data:
        if not _cpu.has_key(_metric['host']):
            _cpu[_metric['host']] = []
        _cpu[_metric['host']].append('%s_%s'%(_metric['timestamp'],_metric['metrics_value']))

    keys = _cpu.keys()
    keys.sort()
    for host_name in keys:
        data_sort = list(set(_cpu[host_name]))
        data_sort.sort()
        data_sort_split = [a.split("_")[1] for a in data_sort][-7:]
        cpu_data_dict['cpus'].append({'name':host_name,'data':data_sort_split})
    cpu_data_json = json.dumps(cpu_data_dict)
    return cpu_data_json

def get_performance_enabled(request):
    settings = vsmapi.get_settings(request)
    perf_settings = [setting for setting in settings if setting.name in ('ceph_diamond_collect_interval', 'cpu_diamond_collect_interval')]
    enabled = any(setting.value != '0' for setting in perf_settings)
    perf_enabled = { "enabled": enabled }
    perf_enabled_data = json.dumps(perf_enabled)
    return perf_enabled_data



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Corporation, All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from datetime import datetime
import logging
import json

from django.core.urlresolvers import reverse, reverse_lazy
from django import http
from django.http import HttpResponse
from django import shortcuts
from django.utils.translation import ugettext_lazy as _
from django.utils.decorators import method_decorator
from django.views.decorators.debug import sensitive_post_parameters

from horizon import exceptions
from horizon import forms
from horizon import tables
from horizon.utils import functions as utils

from vsm_dashboard import api
from .utils import get_admin_tenant
from .tables import ListUserTable
from .forms import CreateUserForm
from .forms import UpdateUserForm
from .forms import UserSettingsForm

LOG = logging.getLogger(__name__)


class IndexView(tables.DataTableView):
    table_class = ListUserTable
    template_name = 'vsm/usermgmt/index.html'

    def get_data(self):

        admin_tenant = get_admin_tenant(self.request)
        _user_list = api.keystone.user_list(self.request, admin_tenant.id)
        LOG.info("USER LIST")
        if self.request.user.username != "admin":
            _user_list = [x for x in _user_list if x.id == self.request.user.id]

        user_list = []
        for _user in _user_list:
            user = {"id": _user.id,
                    "email": _user.email,
                    "enabled": _user.enabled,
                    "name": _user.name,}
            user_list.append(user)
        LOG.info("CEPH_LOG user list: %s" % user_list)

        return user_list

class CreateView(forms.ModalFormView):
    form_class = CreateUserForm
    template_name = 'vsm/usermgmt/create.html'
    success_url = reverse_lazy('horizon:vsm:usermgmt:index')

class UpdateView(forms.ModalFormView):
    form_class = UpdateUserForm
    template_name = 'vsm/usermgmt/update.html'
    success_url = reverse_lazy('horizon:vsm:usermgmt:index')

    @method_decorator(sensitive_post_parameters('password',
                                                'confirm_password'))
    def dispatch(self, *args, **kwargs):
        return super(UpdateView, self).dispatch(*args, **kwargs)

    def get_object(self):
        if not hasattr(self, "_object"):
            try:
                self._object = api.keystone.user_get(self.request,
                                                     self.kwargs['user_id'],
                                                     admin=True)
            except:
                redirect = reverse("horizon:vsm:usermgmt:index")
                exceptions.handle(self.request,
                                  _('Unable to update user.'),
                                  redirect=redirect)
        return self._object

    def get_context_data(self, **kwargs):
        context = super(UpdateView, self).get_context_data(**kwargs)
        context['user'] = self.get_object()
        return context

    def get_initial(self):
        user = self.get_object()
        return {'id': user.id,
                'name': user.name,
                'tenant_id': getattr(user, 'tenantId', None),
                'email': user.email}

    def form_valid(self, form):
        try:
            handled = form.handle(self.request, form.cleaned_data)
        except Exception:
            handled = None
            exceptions.handle(self.request)

        if handled:
            if "HTTP_X_HORIZON_ADD_TO_FIELD" in self.request.META:
                field_id = self.request.META["HTTP_X_HORIZON_ADD_TO_FIELD"]
                data = [self.get_object_id(handled),
                        self.get_object_display(handled)]
                response = http.HttpResponse(json.dumps(data))
                response["X-Horizon-Add-To-Field"] = field_id
            elif isinstance(handled, http.HttpResponse):
                return handled
            else:
                success_url = self.get_success_url()
                response = http.HttpResponseRedirect(success_url)
                # TODO(gabriel): This is not a long-term solution to how
                # AJAX should be handled, but it's an expedient solution
                # until the blueprint for AJAX handling is architected
                # and implemented.
                response['X-Horizon-Location'] = success_url
            return response
        else:
            # If handled didn't return, we can assume something went
            # wrong, and we should send back the form as-is.
            return self.form_invalid(form)



def create_user(request):
    data = json.loads(request.body)

    try:
        admin_tenant = get_admin_tenant(request)
        tenant_id = admin_tenant.id
        #create user
        ret = api.keystone.user_create(request, data['name'], data['email'],
                                            data['pwd'], tenant_id, enabled=True)
        #assign the role
        roles = api.keystone.role_list(request)
        for role in roles:
            if role.name in ['admin', 'KeystoneAdmin']:
                api.keystone.add_tenant_user_role(request, tenant_id, ret.id, role.id)
        
        resp = dict(message="Create user successfully", status="OK", data="")
        resp = json.dumps(resp)
        return HttpResponse(resp)
    except:
        resp = dict(message="Create User failed", status="Bad", data="")
        resp = json.dumps(resp)
        return HttpResponse(resp)


def update_pwd(request):
    data = json.loads(request.body)

    api.keystone.user_update_password(request, data["id"], data["pwd"])
    resp = dict(message="Update User", status="OK", data="")
    resp = json.dumps(resp)
    return HttpResponse(resp)


class UserSettingsView(forms.ModalFormView):
    form_class = UserSettingsForm
    template_name = 'vsm/usermgmt/settings.html'
    success_url = reverse_lazy("horizon:vsm:usermgmt:index")

    def get_initial(self):
        return {
            'pagesize': utils.get_page_size(self.request)
        }

    def form_valid(self, form):
        return form.handle(self.request, form.cleaned_data)

def _one_year():
    now = datetime.utcnow()
    return datetime(now.year + 1, now.month, now.day, now.hour,
                    now.minute, now.second, now.microsecond, now.tzinfo)

def user_setting(request):
    data = json.loads(request.body)
    response = shortcuts.redirect(request.build_absolute_uri())

    request.session['horizon_pagesize'] = data['pagesize']
    response.set_cookie('horizon_pagesize', data['pagesize'],
                        expires=_one_year())

    return response